"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * config.ts
 * Contains the definition of the configuration.
 */
const path_1 = __importDefault(require("path"));
const logger_1 = __importDefault(require("./utils/logger"));
function getEnvVar(name, failIfNotDefined = false) {
    const value = process.env[name];
    if (!value && failIfNotDefined) {
        throw new Error(`"${name}" is not defined.`);
    }
    return value;
}
/**
  * Represents the configuration options of the application.
  * @singleton
  */
class Configuration {
    constructor() {
        const commonEndpointOptions = {
            allowedHttpMethods: new Set(['GET', 'POST']),
            allowedUrlStartsWith: Configuration.applicationUrl,
        };
        const additionalScopes = process.env.ApplicationSecurityAuthAdditionalScopes
            ? process.env.ApplicationSecurityAuthAdditionalScopes
                .split(',')
                .map((value) => value.trim())
                .filter((value) => !!value)
            : [];
        const audience = process.env.ApplicationSecurityAuthAudience || `${Configuration.applicationUrl}/graphql`;
        this.app = {
            port: process.env.PORT || 8080,
            decodeAccessToken: process.env.ApplicationSecurityAuthDecodeAccessToken !== 'N',
            exposeAccessTokenInCookies: process.env.ApplicationSecurityAuthExposeAccessTokenInCookies === 'Y',
        };
        this.oAuth = {
            additionalScopes,
            provider: process.env.ApplicationSecurityAuthProvider || "azure-v1" /* AZURE_V1 */,
            clientId: getEnvVar('ApplicationSecurityAuthClientId', true),
            clientSecret: getEnvVar('ApplicationSecurityAuthClientSecret', true),
            callBackUrl: `${Configuration.applicationUrl}/auth/callback`,
            logoutReturnToUrl: `${Configuration.applicationUrl}/auth/logout/return`,
            logoutUrl: process.env.ApplicationSecurityAuthLogoutUrl,
            metadataUrl: getEnvVar('ApplicationSecurityAuthMetadataUrl', true),
            metadataCacheMaxAge: 3600,
            responseMode: process.env.ApplicationSecurityAuthResponseMode === 'query'
                ? 'query' : 'form_post',
            usernameClaim: process.env.ApplicationSecurityAuthUsernameClaim || 'sub',
            accessTokenUsernameClaim: process.env.ApplicationSecurityAuthAccessTokenUsernameClaim || (process.env.ApplicationSecurityAuthUsernameClaim || 'https://api.workflowgen.com/username'),
            audience,
            tokenValidation: {
                algorithms: new Set(['RS256']),
                audience: getEnvVar('ApplicationSecurityAuthClientId', true),
                clockTolerance: process.env.ApplicationSecurityAuthClockTolerance
                    ? parseInt(process.env.ApplicationSecurityAuthClockTolerance, 10)
                    : 60,
            },
            webviewTokenValidation: {
                algorithms: new Set(['RS256']),
                audience,
                clockTolerance: process.env.ApplicationSecurityAuthClockTolerance
                    ? parseInt(process.env.ApplicationSecurityAuthClockTolerance, 10)
                    : 60,
            },
            jwksClient: {
                rateLimit: true,
                jwksRequestsPerMinute: 120,
                cache: true,
                cacheMaxEntries: 10,
            },
        };
        this.workflowService = {
            debugEnabled: process.env.AuthDebugEnabled
                ? process.env.AuthDebugEnabled === 'Y'
                : false,
            configFilePath: process.env.AuthWfgenConfigFilePath
                ? path_1.default.normalize(process.env.AuthWfgenConfigFilePath)
                : path_1.default.normalize(path_1.default.join(__dirname, '..', '..', 'web.config')),
            serviceAssemblyFilePath: process.env.AuthWfgenAssemblyFilePath
                ? path_1.default.normalize(process.env.AuthWfgenAssemblyFilePath)
                : path_1.default.normalize(path_1.default.join(__dirname, '..', '..', 'bin', 'Advantys.workflow.dll')),
            operationTimeout: process.env.AuthOperationTimeout
                ? parseInt(process.env.AuthOperationTimeout, 10)
                : 30000,
            maxOperationCount: process.env.AuthMaxOperationCount
                ? parseInt(process.env.AuthMaxOperationCount, 10)
                : 10000,
            maxInputFileSize: process.env.AuthMaxInputFileSize
                ? parseInt(process.env.AuthMaxInputFileSize, 10)
                : 10240,
            inputFileDownloadTimeout: process.env.AuthInputFileDownloadTimeout
                ? parseInt(process.env.AuthInputFileDownloadTimeout, 10)
                : 100000,
            maxOperationRetries: process.env.AuthMaxOperationRetries
                ? parseInt(process.env.AuthMaxOperationRetries, 10)
                : 20,
            operationRetryDelay: process.env.AuthOperationRetryDelay
                ? parseInt(process.env.AuthOperationRetryDelay, 10)
                : 200,
            logHandler: (text) => logger_1.default('DEBUG', text),
        };
        this.login = {
            parameters: {
                returnTo: 'redirect_uri',
            },
            ...commonEndpointOptions,
        };
        this.logout = {
            parameters: {
                postLogoutReturnTo: 'post_logout_redirect_uri',
            },
            ...commonEndpointOptions,
        };
        this.session = {
            cookies: {
                token: process.env.ApplicationSecurityAuthSessionTokenCookie || 'wfgen_token',
                idToken: 'wfgen_auth_id_token',
                returnTo: 'wfgen_auth_return_to',
                postLogoutReturnTo: 'wfgen_auth_post_logout_return_to',
                state: 'wfgen_auth_state',
                nonce: 'wfgen_auth_nonce',
                sessionId: 'wfgen_auth_session_id',
                sessionState: 'wfgen_auth_session_state',
                logoutState: 'wfgen_auth_logout_state',
                newSession: 'wfgen_new_session',
                accessToken: 'wfgen_auth_access_token',
            },
            token: {
                signingSecret: getEnvVar('ApplicationSecurityAuthSessiontokenSigningSecret', true),
                issuer: process.env.ApplicationSecurityAuthSessionTokenIssuer || `${Configuration.applicationUrl}/auth`,
                audience: process.env.ApplicationSecurityAuthSessionTokenAudience
                    || Configuration.applicationUrl,
                maxAge: process.env.ApplicationSecurityAuthSessionTimeOut
                    ? parseInt(process.env.ApplicationSecurityAuthSessionTimeOut, 10)
                    : undefined,
                mobileMaxAge: process.env.ApplicationSecurityAuthMobileSessionTimeOut
                    ? parseInt(process.env.ApplicationSecurityAuthMobileSessionTimeOut, 10)
                    : 7200,
            },
        };
        if (process.env.NODE_ENV === 'development') {
            this.debug = {
                returnTo: `${Configuration.applicationUrl}/auth/debug/result`,
                postLogoutReturnTo: `${Configuration.applicationUrl}/auth/debug/logout/result`,
            };
        }
    }
    static get applicationUrl() {
        const applicationUrl = process.env.ApplicationUrl && process.env.ApplicationUrl.endsWith('/')
            ? process.env.ApplicationUrl.substring(0, process.env.ApplicationUrl.length - 1)
            : process.env.ApplicationUrl;
        if (!applicationUrl) {
            throw new Error('"ApplicationUrl" is not defined.');
        }
        return applicationUrl;
    }
    static getInstance() {
        if (!Configuration.instance) {
            Configuration.instance = new Configuration();
        }
        return Configuration.instance;
    }
}
exports.default = Configuration;
