"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable block-scoped-var */
/**
 * oidcController.ts
 * Contains the authentication endpoints.
 */
const buffer_1 = require("buffer");
const crypto = __importStar(require("crypto"));
const express = __importStar(require("express"));
const jsonwebtoken_1 = __importDefault(require("jsonwebtoken"));
const querystring = __importStar(require("querystring"));
const request = __importStar(require("request-promise-native"));
const uuid_1 = require("uuid");
const config_1 = __importDefault(require("../config"));
const token_1 = require("../models/token");
const logger_1 = __importDefault(require("../utils/logger"));
const workflowservice_1 = __importDefault(require("../services/workflow/workflowservice"));
const config = config_1.default.getInstance();
const router = express.Router();
const logDebug = (message) => logger_1.default('DEBUG', message);
const logError = (message) => logger_1.default('ERROR', message);
const logInfo = (message) => logger_1.default('INFO', message);
const httpOnlyCookie = (res, name, value) => {
    if (name === config.session.cookies.sessionId
        || name === config.session.cookies.token
        || name === config.session.cookies.idToken) {
        res.cookie(name, value, { httpOnly: true, sameSite: 'lax', secure: true });
    }
    else {
        res.cookie(name, value, { httpOnly: true, sameSite: 'none', secure: true });
    }
};
const removeCookie = (res, name) => res.clearCookie(name);
const generalGetLogMessage = (reqPath, sessionId, message) => `${reqPath} : ${sessionId} : ${message}`;
const getSessionTokenExpiration = (configOptions, tokenClaims) => {
    if (configOptions.session.token.maxAge
        && configOptions.session.token.maxAge <= tokenClaims.payload.exp) {
        return Math.floor((Date.now() + (configOptions.session.token.maxAge * 1000)) / 1000);
    }
    return tokenClaims.payload.exp;
};
// Login endpoint
async function login(req, res) {
    const state = uuid_1.v4();
    const nonce = uuid_1.v4();
    const sessionId = state; // We use the state as auth session id for the logs
    const getLesserLogMessage = (message) => generalGetLogMessage(req.path, sessionId, message);
    const getLogMessage = (message) => getLesserLogMessage(`${config.login.parameters.returnTo} ${message}`);
    const resCookieHttpOnly = (name, value) => httpOnlyCookie(res, name, value);
    const logLesserDebug = (message) => logDebug(getLesserLogMessage(message));
    const logErrorMessage = (message) => logError(getLogMessage(message));
    logLesserDebug('New authentication request');
    const returnTo = (req.method === 'POST')
        ? req.body[config.login.parameters.returnTo]
        : req.query[config.login.parameters.returnTo];
    const silent = (req.method === 'POST')
        ? req.body.silent
        : req.query.silent;
    let metadata;
    if (!returnTo) {
        logErrorMessage('parameter is required');
        res.status(400).render('error', {
            error: 'missing_parameter',
            errorDescription: `${config.login.parameters.returnTo} parameter is required.`,
        });
        return;
    }
    if (!returnTo.startsWith('/') && !returnTo.startsWith(config.login.allowedUrlStartsWith)) {
        logErrorMessage('parameter value is not authorized');
        res.status(400).render('error', {
            error: 'unauthorized_parameter',
            errorDescription: `${config.login.parameters.returnTo} parameter value is not authorized.`,
        });
        return;
    }
    try {
        metadata = await token_1.getMetadata();
    }
    catch (err) {
        logError(`${req.path} : ${sessionId} : Failed to obtain metadata. Error: ${err}`);
        res.status(400).render('error', {
            error: 'network_error',
            errorDescription: `Failed to obtain metadata. Error: ${err}`,
        });
        return;
    }
    // Set Session cookies
    resCookieHttpOnly(config.session.cookies.returnTo, returnTo);
    resCookieHttpOnly(config.session.cookies.state, state);
    resCookieHttpOnly(config.session.cookies.sessionId, sessionId);
    resCookieHttpOnly(config.session.cookies.nonce, nonce);
    // Initiates an OIDC authorization code grant flow
    // http://openid.net/specs/openid-connect-core-1_0.html#AuthRequest
    const queryObject = {
        scope: ['openid', 'email', 'profile']
            .concat(config.oAuth.additionalScopes)
            .join(' ')
            .trim(),
        response_type: 'code',
        client_id: config.oAuth.clientId,
        redirect_uri: config.oAuth.callBackUrl,
        response_mode: config.oAuth.responseMode,
        audience: config.oAuth.audience,
        state,
        nonce,
    };
    if (config.oAuth.provider === "azure-v1" /* AZURE_V1 */ || config.oAuth.provider === "adfs" /* ADFS */) {
        queryObject.resource = config.oAuth.audience;
    }
    // Silent authentication is used by web apps to refresh the id_token.
    if (silent === 'true') {
        queryObject.prompt = 'none';
        const appToken = req.cookies[config.session.cookies.token];
        if (appToken) {
            const decoded = jsonwebtoken_1.default.decode(appToken, { complete: true });
            if (!decoded || !token_1.Validators.isJWTToken(decoded)) {
                res.status(400).render('error', {
                    error: 'Invalid app token',
                });
                return;
            }
            queryObject.login_hint = decoded.payload.sub;
        }
    }
    const query = querystring.stringify(queryObject);
    const authorizationUrl = `${metadata.authorization_endpoint}?${query}`;
    logLesserDebug(`Login: authorizationUrl: ${authorizationUrl}`);
    // Due to limitations of testing libraries, redirects must be treated
    // differently to get the response correctly.
    /** @see {@link https://github.com/chaijs/chai-http/issues/135} */
    /* istanbul ignore else */
    if (process.env.NODE_ENV === 'test') {
        res.status(200).send(authorizationUrl);
    }
    else {
        res.redirect(302, authorizationUrl);
    }
}
// Handles the authentication response sent by the authorization provider
// http://openid.net/specs/openid-connect-core-1_0.html#AuthResponse
async function loginCallback(req, res) {
    const originalState = req.cookies[config.session.cookies.state];
    const sessionId = req.cookies[config.session.cookies.sessionId];
    const responseData = (config.oAuth.responseMode === 'form_post') ? req.body : req.query;
    const { state, } = responseData;
    const getLogMessage = (message) => generalGetLogMessage(req.path, sessionId || '', message);
    const resCookieHttpOnly = (name, value) => httpOnlyCookie(res, name, value);
    const logThisError = (message) => logError(getLogMessage(message));
    const logThisDebug = (message) => logDebug(getLogMessage(message));
    const logThisInfo = (message) => logInfo(getLogMessage(message));
    let workflowService;
    let metadata;
    try {
        workflowService = new workflowservice_1.default(config.workflowService);
    }
    catch (err) {
        const error = err;
        res.status(400).render('error', {
            error: error.constructor.name,
            errorDescription: error.message,
        });
        return;
    }
    if (originalState !== state) {
        const message = `Callback called with an invalid state: ${state}`;
        logThisError(message);
        res.status(400).render('error', {
            error: 'invalid_state',
            errorDescription: message,
        });
        return;
    }
    if (token_1.Validators.isErrorAuthorizationRequest(responseData)) {
        logThisError(`Failed to obtain an authorization code. Error: ${responseData.error}. ${responseData.error_description}`);
        res.status(400).render('error', {
            error: responseData.error,
            errorDescription: responseData.error_description,
        });
        return;
    }
    try {
        metadata = await token_1.getMetadata();
    }
    catch (err) {
        const message = `Failed to obtain metadata. Error: ${err}`;
        logThisError(message);
        res.status(400).render('error', {
            error: 'network_error',
            errorDescription: message,
        });
        return;
    }
    let body;
    // Request an id_token and an access_token
    // http://openid.net/specs/openid-connect-core-1_0.html#TokenRequest
    try {
        const bodyObject = {
            grant_type: 'authorization_code',
            client_id: config.oAuth.clientId,
            client_secret: config.oAuth.clientSecret,
            code: responseData.code,
            redirect_uri: config.oAuth.callBackUrl,
        };
        if (config.oAuth.provider === "azure-v1" /* AZURE_V1 */ || config.oAuth.provider === "adfs" /* ADFS */) {
            bodyObject.resource = config.oAuth.audience;
        }
        // tslint:disable-next-line: await-promise
        body = await request.post({
            url: metadata.token_endpoint,
            headers: {
                'content-type': 'application/x-www-form-urlencoded',
            },
            body: querystring.stringify(bodyObject),
        });
    }
    catch (err) {
        const message = `Get token operation failed with error: ${err}`;
        logThisError(message);
        res.status(400).render('error', {
            error: 'network_error',
            errorDescription: message,
        });
        return;
    }
    // Token response management
    // http://openid.net/specs/openid-connect-core-1_0.html#TokenResponse
    const responseBody = JSON.parse(body);
    if (token_1.Validators.isErrorTokenResponse(responseBody)) {
        const message = `Get token failed with error: ${responseBody.error} ${responseBody.error_description}`;
        logThisError(message);
        res.status(400).render('error', {
            error: responseBody.error,
            errorDescription: message,
        });
        return;
    }
    const { id_token: idToken, access_token: accessToken, } = responseBody;
    logThisDebug(`Get token succeed: id_token: ${idToken} access_token: ${accessToken}`);
    try {
        // If no id token is returned from the provider, then it is not an OpenID Connect
        // provider and is therefore not supported.
        /* istanbul ignore if */
        if (!idToken) {
            throw new Error('Id token is not present in response.');
        }
        // eslint-disable-next-line no-var, vars-on-top
        var [idTokenClaims, accessTokenClaims = null] = await Promise.all([
            token_1.getTokenClaims(req, idToken, config.oAuth.tokenValidation),
            ...(config.app.decodeAccessToken
                ? [token_1.getTokenClaims(req, accessToken, config.oAuth.webviewTokenValidation)]
                : []),
        ]);
    }
    catch (err) {
        const message = `Failed to get id token claims with error: ${err}`;
        logThisError(message);
        res.status(400).render('error', {
            error: 'get_claims_error',
            errorDescription: message,
        });
        return;
    }
    // Nonce verification
    if (req.cookies[config.session.cookies.nonce] !== idTokenClaims.payload.nonce) {
        const message = `id_token contains an invalid nonce: ${idTokenClaims.payload.nonce}`;
        logThisError(message);
        res.status(400).render('error', {
            error: 'invalid_nonce',
            errorDescription: message,
        });
        return;
    }
    // Access token validation
    /** @see {@link https://openid.net/specs/openid-connect-core-1_0.html#ImplicitTokenValidation} */
    if (idTokenClaims.payload.at_hash) {
        const accessTokenHash = crypto
            .createHash('sha256')
            .update(accessToken, 'utf8')
            .digest('hex');
        const leftPart = accessTokenHash.substr(0, accessTokenHash.length / 2);
        const leftPartBase64UrlEncoded = buffer_1.Buffer.from(leftPart, 'hex')
            .toString('base64')
            .replace(/\+/g, '-')
            .replace(/\//g, '_')
            .replace(/=/g, '');
        logThisDebug(`at_hash check for value: ${idTokenClaims.payload.at_hash} against: ${leftPartBase64UrlEncoded}`);
        if (leftPartBase64UrlEncoded !== idTokenClaims.payload.at_hash) {
            logThisError('Access token not valid after at_hash verification. '
                + `at_hash: ${idTokenClaims.payload.at_hash}. encoded access token: ${leftPartBase64UrlEncoded}`);
            res.status(400).render('error', {
                error: 'invalid_at_hash',
                errorDescription: 'Invalid access token provided with id token.',
            });
            return;
        }
    }
    logThisDebug('id_token validation succeed');
    const usernameClaim = idTokenClaims.payload[config.oAuth.usernameClaim];
    let encryptedAccessToken;
    // We ignore the else path because these tests would require a more complexe
    // stub system to verify the console logs.
    /* istanbul ignore if */
    if (!usernameClaim) {
        logThisInfo('The username claim is empty. Your configuration may not specify the correct claim to take.');
    }
    // Securely store the access token in the database
    try {
        const { encryptedAccessToken: resultAccessToken } = await workflowService.invoke('updateViewerAccessToken', { authUserName: usernameClaim }, {
            input: { accessToken },
        });
        encryptedAccessToken = resultAccessToken;
    }
    catch (err) {
        const error = err;
        if (error.name === 'System.UnauthorizedAccessException') {
            res.status(401).render('error', {
                error: error.name,
                errorDescription: error.message,
            });
            return;
        }
        logError(`An error occured in the facade: ${error.message}`);
        res.status(400).render('error', {
            error: 'Authentication Error',
            errorDescription: 'An unexpected error occured during the authentication process.',
        });
        return;
    }
    let appToken;
    // Create Application session JWT token
    try {
        const { sub, iss, aud, exp, iat, ...otherClaims } = idTokenClaims.payload;
        appToken = jsonwebtoken_1.default.sign({
            sub: usernameClaim || '',
            iss: config.session.token.issuer,
            aud: config.session.token.audience,
            exp: getSessionTokenExpiration(config, accessTokenClaims || idTokenClaims),
            ...otherClaims,
        }, config.session.token.signingSecret);
    }
    catch (err) {
        const message = `Cannot sign a new session token: ${err}`;
        logThisError(message);
        res.status(400).render('error', {
            error: 'signing_error',
            errorDescription: message,
        });
        return;
    }
    logThisDebug(`Application session token created. sub: ${usernameClaim} id_token: ${appToken}`);
    const returnTo = req.cookies[config.session.cookies.returnTo];
    if (!returnTo) {
        const message = 'The returnTo value is missing from the cookies.';
        logThisError(message);
        res.status(400).render('error', {
            error: 'redirect_error',
            errorDescription: message,
        });
        return;
    }
    // Set application token session cookie
    resCookieHttpOnly(config.session.cookies.token, appToken);
    resCookieHttpOnly(config.session.cookies.newSession, 'true');
    if (config.app.exposeAccessTokenInCookies) {
        resCookieHttpOnly(config.session.cookies.accessToken, encryptedAccessToken);
    }
    // Set session_state cookie if supported by the provider
    // http://openid.net/specs/openid-connect-session-1_0.html
    if (responseData.session_state) {
        resCookieHttpOnly(config.session.cookies.sessionState, responseData.session_state);
    }
    if (config.oAuth.provider === "okta" /* OKTA */
        || config.oAuth.provider === "azure-v1" /* AZURE_V1 */
        || config.oAuth.provider === "ms-identity-v2" /* MS_IDENTITY_V2 */) {
        resCookieHttpOnly(config.session.cookies.idToken, idToken);
    }
    // Remove useless session cookies
    removeCookie(res, config.session.cookies.returnTo);
    removeCookie(res, config.session.cookies.nonce);
    removeCookie(res, config.session.cookies.state);
    /* istanbul ignore else */
    if (process.env.NODE_ENV === 'test') {
        res.status(200).send(returnTo);
    }
    else {
        res.redirect(returnTo);
    }
}
// Initiates a logout
async function logout(req, res) {
    const sessionId = req.cookies[config.session.cookies.sessionId];
    const getLogMessage = (message) => generalGetLogMessage(req.path, sessionId || '', message);
    const resCookieHttpOnly = (name, value) => httpOnlyCookie(res, name, value);
    const logThisDebug = (message) => logDebug(getLogMessage(message));
    const logThisError = (message) => logError(getLogMessage(message));
    const logThisInfo = (message) => logInfo(getLogMessage(message));
    let metadata;
    logThisDebug('Initiates a logout');
    const postLogoutReturnTo = req.method === 'POST'
        ? req.body[config.logout.parameters.postLogoutReturnTo]
        : req.query[config.logout.parameters.postLogoutReturnTo];
    if (config.oAuth.provider !== "adfs" /* ADFS */) {
        if (!postLogoutReturnTo) {
            const message = `${config.logout.parameters.postLogoutReturnTo} parameter is required`;
            logThisError(message);
            res.status(400).render('error', {
                error: 'required_parameter',
                errorDescription: message,
            });
            return;
        }
        if (!postLogoutReturnTo.startsWith('/') && !postLogoutReturnTo.startsWith(config.logout.allowedUrlStartsWith)) {
            const message = `${config.logout.parameters.postLogoutReturnTo} parameter value is not authorized`;
            logThisError(message);
            res.status(400).render('error', {
                error: 'unauthorized_parameter',
                errorDescription: message,
            });
            return;
        }
    }
    try {
        metadata = await token_1.getMetadata();
    }
    catch (err) {
        const message = `Failed to obtain metadata. Error: ${err}`;
        logThisError(message);
        res.status(400).render('error', {
            error: 'network_error',
            errorDescription: message,
        });
        return;
    }
    let logoutUrl;
    const logoutState = uuid_1.v4();
    let query;
    // If a new provider have to be added, encapsulate specific functionalities into
    // a conception pattern like strategy and factory.
    switch (config.oAuth.provider) {
        case "azure-v1" /* AZURE_V1 */:
        case "ms-identity-v2" /* MS_IDENTITY_V2 */:
        case "okta" /* OKTA */:
            query = querystring.stringify({
                post_logout_redirect_uri: config.oAuth.logoutReturnToUrl,
                id_token_hint: req.cookies[config.session.cookies.idToken],
                state: logoutState,
            });
            logoutUrl = `${metadata.end_session_endpoint}?${query}`;
            resCookieHttpOnly(config.session.cookies.logoutState, logoutState);
            break;
        case "adfs" /* ADFS */:
            logoutUrl = metadata.end_session_endpoint;
            // Since ADFS doesn't support the redirection of the user to the app
            // after the logout, we immediatly clear the session cookies before redirecting
            // the user to the ADFS logout.
            removeCookie(res, config.session.cookies.token);
            removeCookie(res, config.session.cookies.sessionId);
            removeCookie(res, config.session.cookies.sessionState);
            removeCookie(res, config.session.cookies.newSession);
            break;
        default: // Other providers
            query = querystring.stringify({
                client_id: config.oAuth.clientId,
                returnTo: config.oAuth.logoutReturnToUrl,
            });
            logoutUrl = `${config.oAuth.logoutUrl}?${query}`; // For Auth0
            break;
    }
    if (postLogoutReturnTo) {
        resCookieHttpOnly(config.session.cookies.postLogoutReturnTo, postLogoutReturnTo);
    }
    logThisInfo(`Redirection to: ${logoutUrl}`);
    // istanbul ignore else
    if (process.env.NODE_ENV === 'test') {
        res.status(200).send(logoutUrl);
    }
    else {
        res.redirect(logoutUrl);
    }
}
// Handles the logout return
function logoutReturn(req, res) {
    const sessionId = req.cookies[config.session.cookies.sessionId];
    const logoutState = req.cookies[config.session.cookies.logoutState];
    const { state } = req.query;
    const getLogMessage = (message) => generalGetLogMessage(req.path, sessionId || '', message);
    const logThisInfo = (message) => logInfo(getLogMessage(message));
    const logThisError = (message) => logError(getLogMessage(message));
    logThisInfo(`Request for post logout from origin: ${req.protocol}://${req.hostname}`);
    // Check logoutState for OpenID Connect session management compliant providers
    if (logoutState && logoutState !== state) {
        const message = `Logout callback called with an invalid state: ${state}`;
        logThisError(message);
        res.status(400).send(message);
        return;
    }
    logThisInfo('Authentication Provider logout done.');
    // Cookies clean-up
    removeCookie(res, config.session.cookies.token);
    removeCookie(res, config.session.cookies.idToken);
    removeCookie(res, config.session.cookies.sessionId);
    removeCookie(res, config.session.cookies.sessionState);
    removeCookie(res, config.session.cookies.postLogoutReturnTo);
    removeCookie(res, config.session.cookies.logoutState);
    removeCookie(res, config.session.cookies.newSession);
    // eslint-disable-next-line max-len
    const postLogoutReturnTo = req.cookies[config.session.cookies.postLogoutReturnTo];
    if (!postLogoutReturnTo) {
        const message = 'The "postLogoutReturnTo" value is missing from the cookies.';
        logThisError(message);
        res.render('error', {
            error: 'missing_cookie',
            errorDescription: message,
        });
    }
    /* istanbul ignore else */
    if (process.env.NODE_ENV === 'test') {
        res.status(200).send(postLogoutReturnTo);
    }
    else {
        res.redirect(postLogoutReturnTo);
    }
}
// Set Routes
if (config.login.allowedHttpMethods.has('GET')) {
    router.get('/', login);
}
if (config.login.allowedHttpMethods.has('POST')) {
    router.post('/', login);
}
if (config.logout.allowedHttpMethods.has('GET')) {
    router.get('/logout', logout);
}
if (config.logout.allowedHttpMethods.has('POST')) {
    router.post('/logout', logout);
}
if (config.oAuth.responseMode === 'form_post') {
    router.post('/callback', loginCallback);
}
else {
    router.get('/callback', loginCallback);
}
if (config.oAuth.provider !== "adfs" /* ADFS */) {
    router.get('/logout/return', logoutReturn);
}
exports.default = router;
