"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTokenClaims = exports.getSigningKey = exports.getBearerToken = exports.getMetadata = exports.Validators = void 0;
/**
 * token.ts
 * Contains the JWT Token management.
 */
const got_1 = __importDefault(require("got"));
const jsonwebtoken_1 = __importDefault(require("jsonwebtoken"));
const jwks_rsa_1 = __importDefault(require("jwks-rsa"));
const mem_1 = __importDefault(require("mem"));
const config_1 = __importDefault(require("../config"));
const logger_1 = __importDefault(require("../utils/logger"));
const config = config_1.default.getInstance();
var Validators;
(function (Validators) {
    function isJWTToken(object) {
        if (config.oAuth.provider === "adfs" /* ADFS */) { // No sub claim in access token for ADFS
            return 'payload' in object
                && 'header' in object
                && 'alg' in object.header
                && 'exp' in object.payload
                && 'aud' in object.payload;
        }
        return 'payload' in object
            && 'header' in object
            && 'alg' in object.header
            && 'exp' in object.payload
            && 'aud' in object.payload
            && 'sub' in object.payload;
    }
    Validators.isJWTToken = isJWTToken;
    function isSuccessfulAuthorizationRequest(object) {
        return 'code' in object;
    }
    Validators.isSuccessfulAuthorizationRequest = isSuccessfulAuthorizationRequest;
    function isErrorAuthorizationRequest(object) {
        return 'error' in object;
    }
    Validators.isErrorAuthorizationRequest = isErrorAuthorizationRequest;
    function isSuccessfulTokenResponse(object) {
        return 'access_token' in object
            && 'expires_in' in object
            && 'token_type' in object
            && object.token_type === 'Bearer';
    }
    Validators.isSuccessfulTokenResponse = isSuccessfulTokenResponse;
    function isErrorTokenResponse(object) {
        return 'error' in object;
    }
    Validators.isErrorTokenResponse = isErrorTokenResponse;
})(Validators = exports.Validators || (exports.Validators = {}));
class CustomError extends Error {
    constructor(message, originalError) {
        super(message);
        this.originalError = originalError;
    }
}
exports.getMetadata = mem_1.default(() => got_1.default(config.oAuth.metadataUrl)
    .then((result) => JSON.parse(result.body)), {
    maxAge: config.oAuth.metadataCacheMaxAge * 1000,
});
const logError = (message) => logger_1.default('ERROR', message);
function getBearerToken(req) {
    const { authorization } = req.headers;
    if (!authorization) {
        return null;
    }
    const parts = authorization.split(' ');
    const [tokenType, token] = parts;
    return !(parts.length === 2 && tokenType === 'Bearer' && token)
        ? null
        : token;
}
exports.getBearerToken = getBearerToken;
/**
 * Get the signing key for the received token.
 */
function getSigningKey(kid, jwksClientOptions) {
    return new Promise((resolve, reject) => {
        const client = jwks_rsa_1.default(jwksClientOptions);
        const getKeys = client.getKeys.bind(client);
        // Workaround proxy to get the certificate because jwks-rsa doesn't support getting
        // the key when there's only an x5t value and no kid yet.
        client.getKeys = function (cb) {
            getKeys((err, keys) => {
                if (err) {
                    cb(err, keys);
                    return;
                }
                const proxiedKeys = keys
                    .map((k) => {
                    if (!k.kid && k.x5t) {
                        // eslint-disable-next-line no-param-reassign
                        k.kid = k.x5t;
                    }
                    return k;
                });
                cb(err, proxiedKeys);
            });
        };
        client.getSigningKey(kid, (err, key) => {
            if (err) {
                reject(err);
                return;
            }
            resolve(key.publicKey || key.rsaPublicKey);
        });
    });
}
exports.getSigningKey = getSigningKey;
/**
 * Get the JWT claims
 */
async function getTokenClaims(req, token, tokenValidationOptions) {
    const getLogMessage = (message) => `${req.path} : ${message}`;
    const logThisError = (message) => logError(getLogMessage(message));
    const metadata = await exports.getMetadata();
    const jwksClientOptions = Object.assign(config.oAuth.jwksClient, {
        jwksUri: metadata.jwks_uri,
    });
    let tokenClaims;
    try {
        tokenClaims = jsonwebtoken_1.default.decode(token, { complete: true });
        if (!tokenClaims || !Validators.isJWTToken(tokenClaims)) {
            throw new Error('Invalid token decoded.');
        }
    }
    catch (err) {
        const error = new CustomError(`Invalid bearer token. Decoding error : ${err}`, err);
        logThisError(error.message);
        throw error;
    }
    let key;
    try {
        key = await getSigningKey(tokenClaims.header.kid ?? tokenClaims.header.x5t ?? '', jwksClientOptions);
        if (!key) {
            throw new Error('The signing key of the token cannot be found.');
        }
    }
    catch (err) {
        const error = new CustomError(`Token validation error. Failed to get the signing key with error : ${err}`, err);
        logThisError(error.message);
        throw error;
    }
    try {
        // Audience validation is done in this function
        jsonwebtoken_1.default.verify(token, key, Object.assign(tokenValidationOptions, {
            algorithms: Array.from(tokenValidationOptions.algorithms),
        }));
    }
    catch (err) {
        const error = new CustomError(`Token validation error : ${err}`, err);
        logThisError(error.message);
        throw error;
    }
    return tokenClaims;
}
exports.getTokenClaims = getTokenClaims;
