"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * config.ts
 * This file contains the initialization of the configuration.
 */
const path = __importStar(require("path"));
const logger_1 = require("./utils/logger");
/**
 * Configuration class for the application.
 * @singleton
 */
class Configuration {
    constructor() {
        this.applicationDataPath = process.env.ApplicationDataPath ||
            path.normalize(path.join(__dirname, '..', 'App_data'));
        const debugEnabled = getEnvVar('GraphqlDebugEnabled', false, {
            boolean: {
                operator: 'eq',
                comparisonValue: 'Y'
            }
        });
        const fileUploadPath = path.join(this.applicationDataPath, 'Files', 'GraphQL', 'Uploads');
        const inputFileAllowedFolders = getEnvVar('GraphqlInputFileAllowedFolders', '');
        const getInputFileAllowedFolders = () => {
            let allowedFolders = inputFileAllowedFolders;
            if (!allowedFolders) {
                allowedFolders = fileUploadPath;
            }
            else if (!inputFileAllowedFolders.includes(fileUploadPath)) {
                allowedFolders += `,${fileUploadPath}`;
            }
            return allowedFolders;
        };
        this.app = {
            port: getEnvVar('PORT', '8080')
        };
        this.graphql = {
            graphiqlEnabled: getEnvVar('GraphqlGraphiqlEnabled', true, {
                boolean: {
                    operator: 'eq',
                    comparisonValue: 'Y'
                }
            }),
            mocksEnabled: false,
            debugEnabled,
            queryTimeout: getEnvVar('GraphqlQueryTimeout', 90000),
            maxRequestBodySize: process.env.NODE_ENV === 'test' ? 5 : 100,
            maxUploadFiles: 30,
            fileUploadPath: process.env.NODE_ENV === 'test' ? 'Uploads/' : fileUploadPath,
            apiKeyEnabled: getEnvVar('GraphqlApiKeyEnabled', false, {
                boolean: {
                    operator: 'eq',
                    comparisonValue: 'Y'
                }
            }),
            apiKey: getEnvVar('GraphqlApiKey')
        };
        this.workflowService = {
            debugEnabled,
            configFilePath: path.normalize(path.join(__dirname, '..', '..', 'web.config')),
            serviceAssemblyFilePath: getEnvVar('AdvantysWorkflowDllPath', path.normalize(path.join(__dirname, '..', '..', 'bin', 'Advantys.workflow.dll'))),
            logHandler: (text) => logger_1.logger('DEBUG', text),
            maxPageSize: getEnvVar('GraphqlMaxPageSize', 100),
            maxBatchSize: getEnvVar('GraphqlMaxBatchSize', 1000),
            defaultPageSize: getEnvVar('GraphqlDefaultPageSize', 30),
            defaultPageNumber: getEnvVar('GraphqlDefaultPageNumber', 1),
            operationTimeout: getEnvVar('GraphqlOperationTimeout', 30000),
            maxOperationCount: getEnvVar('GraphqlMaxOperationCount', 10000),
            maxInputFileSize: getEnvVar('GraphqlMaxInputFileSize', 10240),
            maxInputFileContentSize: getEnvVar('GraphqlMaxInputFileContentSize', 0, {
                env: [
                    {
                        name: 'test',
                        defaultValue: 1
                    }
                ]
            }),
            inputFileDownloadTimeout: getEnvVar('GraphqlInputFileDownloadTimeout', 100000),
            maxOperationRetries: getEnvVar('GraphqlMaxOperationRetries', 20),
            operationRetryDelay: getEnvVar('GraphqlOperationRetryDelay', 200),
            inputFileAllowedFolders: getInputFileAllowedFolders(),
            sendFileBufferSize: getEnvVar('GraphqlSendFileBufferSize', 80400),
            inputFileAllowedHttpUrls: getEnvVar('GraphqlInputFileAllowedHttpUrls', '')
        };
        this.loaders = {
            cacheEnabled: getEnvVar('GraphqlLoadersCacheEnabled', true, {
                boolean: {
                    operator: 'eq',
                    comparisonValue: 'Y'
                }
            }),
            batchEnabled: getEnvVar('GraphqlLoadersBatchEnabled', true, {
                boolean: {
                    operator: 'eq',
                    comparisonValue: 'Y'
                }
            }),
            maxBatchSize: this.workflowService.maxBatchSize
        };
        this.oAuth = {
            metadataUrl: getEnvVar('ApplicationSecurityAuthMetadataUrl'),
            metadataCacheMaxAge: 3600,
            accessTokenUsernameClaim: getEnvVar('ApplicationSecurityAuthAccessTokenUsernameClaim', getEnvVar('ApplicationSecurityAuthUsernameClaim', 'https://api.workflowgen.com/username')),
            accessTokenAppIdClaim: getEnvVar('ApplicationSecurityAuthAppIdClaim', 'sub'),
            accessTokenProvider: getEnvVar('ApplicationSecurityAuthProvider'),
            graphqlScope: getEnvVar('ApplicationSecurityAuthGraphQLScope'),
            graphqlAppRole: getEnvVar('ApplicationSecurityAuthGraphQLAppRole'),
            tokenValidation: {
                algorithms: new Set(['RS256']),
                audience: getEnvVar('ApplicationSecurityAuthAudience', `${this.applicationUrl}/graphql`),
                clockTolerance: getEnvVar('ApplicationSecurityAuthClockTolerance', 60)
            },
            jwksClient: {
                rateLimit: true,
                jwksRequestsPerMinute: 120,
                cache: true,
                cacheMaxEntries: 10
            }
        };
        if (process.env.NODE_ENV === 'test') {
            this.test = {
                endpoints: {
                    graphql: getEnvVar('GraphqlEndpoint', '/wfgen/graphql/'),
                    fileDownload: getEnvVar('FileDownloadEndpoint', '/wfgen/graphql/workflow/data/files/')
                }
            };
        }
    }
    get applicationUrl() {
        const applicationUrl = process.env.ApplicationUrl && process.env.ApplicationUrl.endsWith('/')
            ? process.env.ApplicationUrl.substring(0, process.env.ApplicationUrl.length - 1)
            : process.env.ApplicationUrl;
        if (!applicationUrl) {
            throw new Error('"ApplicationUrl" is not defined.');
        }
        return applicationUrl;
    }
    static getInstance() {
        if (!this.instance) {
            this.instance = new Configuration();
        }
        return this.instance;
    }
}
exports.default = Configuration;
function getEnvVar(name, defaultValue, options) {
    const envVar = process.env[name];
    if (typeof defaultValue === 'undefined') {
        return envVar;
    }
    if (typeof envVar === 'undefined') {
        if (options && options.env) {
            const env = options.env.find(env => env.name === process.env.NODE_ENV);
            if (env) {
                return env.defaultValue;
            }
        }
        return defaultValue;
    }
    switch (typeof defaultValue) {
        case 'string':
            return envVar;
        case 'number':
            return defaultValue % 1 === 0
                ? parseInt(envVar, undefined)
                : parseFloat(envVar);
        case 'boolean':
            if (options && options.boolean) {
                switch (options.boolean.operator) {
                    case 'eq':
                        return envVar === options.boolean.comparisonValue;
                    case 'ne':
                        return envVar === options.boolean.comparisonValue;
                }
            }
            return Boolean(envVar);
    }
}
