"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const config_1 = __importDefault(require("../../config"));
const token_1 = require("./token");
const config = config_1.default.getInstance();
/**
 * Authenticate the request
 * @param request A request.
 * @return A promise.
 */
function authenticate(request) {
    return __awaiter(this, void 0, void 0, function* () {
        // Retrieve the authenticated username from iisnode promoted server variables
        // Authentication methods:
        // - Basic
        // - Windows
        // - .Net Http authentication module setting the user principal.
        // Security: in IIS, the application authentication MUST have anonymous access disabled
        // or an Http module is configured to define the user principal.
        if (request.headers['x-iisnode-auth_user']) {
            const userName = request.headers['x-iisnode-auth_user'];
            request.userName = userName;
            request.impersonateUserName = getImpersonateUserName(request);
            return userName;
        }
        // Authorization with a Bearer token
        if (request.headers.authorization) {
            const token = token_1.getBearerToken(request);
            if (!token) {
                throw new Error('Only a Bearer token is supported in the authorization header');
            }
            const { claims, isClientCredentials } = yield token_1.getTokenClaims(request, token, config.oAuth.tokenValidation);
            if (!config.oAuth.accessTokenProvider) {
                throw new Error('"accessTokenProvider" is not defined.');
            }
            if (isClientCredentials) {
                request.appClientId = claims.payload[config.oAuth.accessTokenAppIdClaim];
            }
            else {
                request.userName = claims.payload[config.oAuth.accessTokenUsernameClaim];
                request.impersonateUserName = getImpersonateUserName(request);
            }
            return claims.payload.sub;
        }
        // Use env variable in development only
        if ((process.env.NODE_ENV === 'development') && process.env.AUTH_USER) {
            request.userName = process.env.AUTH_USER;
            return process.env.AUTH_USER;
        }
        throw new Error('No valid authentication information found');
    });
}
exports.authenticate = authenticate;
/**
 * Get the impersonate username.
 * @param request A request.
 * @return The username.
 */
function getImpersonateUserName(request) {
    const impersonateUserNameHeader = process.env.GraphqlImpersonateUserNameHttpHeader || 'x-wfgen-impersonate-username';
    const userName = request.headers[impersonateUserNameHeader] || process.env.IMPERSONATE_USER;
    return userName ? userName.toLowerCase() : undefined;
}
exports.getImpersonateUserName = getImpersonateUserName;
/**
 * Creates a workflow context.
 * @param request A request.
 * @return The workflow context.
 */
function getWorkflowContext(request) {
    const workflowContext = {
        operationCount: 0,
        operationErrorCount: 0
    };
    if (request.userName) {
        workflowContext.authUserName = request.userName;
    }
    if (request.impersonateUserName) {
        workflowContext.impersonateUserName = request.impersonateUserName;
    }
    if (request.appClientId) {
        workflowContext.appClientId = request.appClientId;
    }
    return workflowContext;
}
exports.getWorkflowContext = getWorkflowContext;
