"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * token.ts
 * Contains the JWT Token management.
 */
const got_1 = __importDefault(require("got"));
const jwks_rsa_1 = __importDefault(require("jwks-rsa"));
const mem_1 = __importDefault(require("mem"));
const jsonwebtoken_1 = __importDefault(require("jsonwebtoken"));
const config_1 = __importDefault(require("../../config"));
const accesstokenrecipientvalidator_1 = require("./accesstokenrecipientvalidator");
const logger_1 = require("../../utils/logger");
const ADFS_CLIENT_CREDENTIALS_AUTH_METHOD = 'http://schemas.microsoft.com/ws/2008/06/identity/authenticationmethod/password';
var Validators;
(function (Validators) {
    function isJWTToken(object) {
        if (config.oAuth.accessTokenProvider === "adfs" /* ADFS */) { // No sub claim in access token for ADFS
            return 'payload' in object &&
                'header' in object &&
                'alg' in object.header &&
                'exp' in object.payload &&
                'aud' in object.payload;
        }
        return 'payload' in object &&
            'header' in object &&
            'alg' in object.header &&
            'exp' in object.payload &&
            'aud' in object.payload &&
            'sub' in object.payload;
    }
    Validators.isJWTToken = isJWTToken;
    function isSuccessfulAuthorizationRequest(object) {
        return 'code' in object;
    }
    Validators.isSuccessfulAuthorizationRequest = isSuccessfulAuthorizationRequest;
    function isErrorAuthorizationRequest(object) {
        return 'error' in object;
    }
    Validators.isErrorAuthorizationRequest = isErrorAuthorizationRequest;
    function isSuccessfulTokenResponse(object) {
        return 'access_token' in object &&
            'expires_in' in object &&
            'token_type' in object &&
            object.token_type === 'Bearer';
    }
    Validators.isSuccessfulTokenResponse = isSuccessfulTokenResponse;
    function isErrorTokenResponse(object) {
        return 'error' in object;
    }
    Validators.isErrorTokenResponse = isErrorTokenResponse;
})(Validators = exports.Validators || (exports.Validators = {}));
class CustomError extends Error {
    constructor(message, originalError) {
        super(message);
        this.originalError = originalError;
    }
}
const config = config_1.default.getInstance();
exports.getMetadata = mem_1.default(() => got_1.default(config.oAuth.metadataUrl)
    .then(result => JSON.parse(result.body)), {
    maxAge: config.oAuth.metadataCacheMaxAge * 1000
});
const logError = (message) => logger_1.logger('ERROR', message);
function getBearerToken(req) {
    const { authorization } = req.headers;
    if (!authorization) {
        return null;
    }
    const parts = authorization.split(' ');
    const [tokenType, token] = parts;
    return !(parts.length === 2 && tokenType === 'Bearer' && token)
        ? null
        : token;
}
exports.getBearerToken = getBearerToken;
/**
 * Get the JWT claims
 */
function getTokenClaims(req, token, tokenValidationOptions) {
    return __awaiter(this, void 0, void 0, function* () {
        const getLogMessage = (message) => `${req.path} : ${message}`;
        const logThisError = (message) => logError(getLogMessage(message));
        const metadata = yield exports.getMetadata();
        const jwksClientOptions = Object.assign(config.oAuth.jwksClient, {
            jwksUri: metadata.jwks_uri
        });
        try {
            var tokenClaims = jsonwebtoken_1.default.decode(token, { complete: true });
            if (!tokenClaims || !Validators.isJWTToken(tokenClaims)) {
                throw new Error('Invalid token decoded.');
            }
        }
        catch (err) {
            const error = new CustomError(`Invalid bearer token. Decoding error : ${err}`, err);
            logThisError(error.message);
            throw error;
        }
        try {
            var key = yield getSigningKey(tokenClaims.header.kid || tokenClaims.header.x5t || '', jwksClientOptions);
            if (!key) {
                throw new Error('The signing key of the token cannot be found.');
            }
        }
        catch (err) {
            const error = new CustomError(`Token validation error. Failed to get the signing key with error : ${err}`, err);
            logThisError(error.message);
            throw error;
        }
        try {
            // audience validation is done in this function
            jsonwebtoken_1.default.verify(token, key, Object.assign(tokenValidationOptions, {
                algorithms: Array.from(tokenValidationOptions.algorithms)
            }));
        }
        catch (err) {
            const error = new CustomError(`Token validation error : ${err}`, err);
            logThisError(error.message);
            throw error;
        }
        // The API can receive two types of access_token:
        // - one generated by an client-credentials flow (daemons)
        // - one generated by an interactive flow (a user)
        const accessTokenProvider = config.oAuth.accessTokenProvider;
        const isClientCredentials = (accessTokenProvider === "auth0" /* AUTH0 */ && tokenClaims.payload.gty === 'client-credentials') ||
            ((accessTokenProvider === "azure-v1" /* AZURE_V1 */ ||
                accessTokenProvider === "ms-identity-v2" /* MS_IDENTITY_V2 */) &&
                !tokenClaims.payload.scp) ||
            (accessTokenProvider === "adfs" /* ADFS */ && tokenClaims.payload.authmethod === ADFS_CLIENT_CREDENTIALS_AUTH_METHOD) ||
            (accessTokenProvider === "okta" /* OKTA */ && !(tokenClaims.payload[config.oAuth.accessTokenUsernameClaim]));
        // Verifies the access token recipient. If it doesn't target GraphQL, the request is rejected.
        // The token can target the GraphQL API directly by its audience or indirectly, as a workaround,
        // by its scopes.
        if (!accesstokenrecipientvalidator_1.AccessTokenRecipentValidatorFactory.getDefault().validate(tokenClaims, config, isClientCredentials)) {
            const message = 'Cannot validate the audience of the token';
            logThisError(message);
            throw new Error(message);
        }
        return {
            claims: tokenClaims,
            isClientCredentials
        };
    });
}
exports.getTokenClaims = getTokenClaims;
/**
 * Get the signing key for the received token.
 */
function getSigningKey(kid, jwksClientOptions) {
    return new Promise((resolve, reject) => {
        const client = jwks_rsa_1.default(jwksClientOptions);
        const getKeys = client.getKeys.bind(client);
        // Workaround proxy to get the certificate because jwks-rsa doesn't support getting
        // the key when there's only an x5t value and no kid yet.
        client.getKeys = function (cb) {
            getKeys((err, keys) => {
                if (err) {
                    cb(err, keys);
                    return;
                }
                const proxiedKeys = keys
                    .map((k) => {
                    if (!k.kid && k.x5t) {
                        k.kid = k.x5t;
                    }
                    return k;
                });
                cb(err, proxiedKeys);
            });
        };
        client.getSigningKey(kid, (err, key) => {
            if (err) {
                reject(err);
                return;
            }
            resolve(key.publicKey || key.rsaPublicKey);
        });
    });
}
exports.getSigningKey = getSigningKey;
