"use strict";
/**
 * application.ts
 * GraphQL Type definitions using schema language
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = `

# An application.
type Application implements Node {

  # Application assembly class name.
  assemblyClassName: String

  # Application assembly name.
  assemblyName: String

  # Application client identifier.
  clientId: String

  # Application context format.
  contextFormat: ApplicationContextFormatType

  # Application content type.
  contentType: String

  # Application description.
  description: String!

  # Application identifier.
  id: ID!

  # Indicates if the application is active.
  isActive: Boolean!

  # Indicates if the application is the default one.
  isDefault: Boolean!

  # Indicates if the application schema definition is embedded.
  isSchemaDefinitionEmbedded: Boolean

  # Application method.
  method: String

  # Application name.
  name: String!

  # Application parameters.
  parameters(orderBy: ApplicationParameterListOrder) : ApplicationParameterListPage

  # Application password.
  password: String

  # Application token.
  token: String

  # Application type.
  type: ApplicationType!

  # Application URL.
  url: String

  # Application username.
  username: String

  # Application WSDL.
  wsdl: String
}

# Represent the application types.
enum ApplicationType {

 # Assembly type application.
 ASSEMBLY

 # Asynchronous web procedure type application.
 WEBPROCASYNC

 # Incoming webhook type application.
 INWEBHOOK

 # SOAP web service type application.
 WEBSERVICE

 # WCF service type application.
 WCFSERVICE

 # Web application type application.
 WEBAPP

 # Web procedure type application.
 WEBPROC

 # Non interactive client type application.
 NONINTERACTIVECLIENT
}

# Represents the application context format types.
enum ApplicationContextFormatType {

  # Application JSON context format type.
  JSON

  # Application XML ADO.NET DataSet context format type.
  WFCONTEXT
}

# An application parameter.
type ApplicationParameter implements Node {

  # Application parameter data type.
  dataType: WorkflowDataType!

  # Application parameter description.
  description: String!

  # Application parameter direction.
  direction: ParameterDirection!

  # Application parameter identifier.
  id: ID!

  # Application parameter default.
  isDefault: Boolean!

  # Indicates if the application parameter is required.
  isRequired: Boolean!

  # Application parameter name.
  name: String!
}

# A paginated list of application parameters.
type ApplicationParameterListPage {

  # Identifies the total count of items in the page.
  totalCount: Int!

  # Indicates if there are more pages to fetch.
  hasNextPage: Boolean!

  # Indicates if there are any pages prior to the current page.
  hasPreviousPage: Boolean!

  # List of application parameters.
  items: [ApplicationParameter]
}

# A paginated list of applications.
type ApplicationListPage {

  # Identifies the total count of items in the page.
  totalCount: Int!

  # Indicates if there are more pages to fetch.
  hasNextPage: Boolean!

  # Indicates if there are any pages prior to the current page.
  hasPreviousPage: Boolean!

  # List of applications.
  items: [Application]
}

# A filter on applications.
input ApplicationFilter {

  # Filter application by description.
  descriptionContains: String

  # Indicate if the application is active.
  isActive: Boolean

  # Filter application by name.
  nameContains: String

  # Filter application by type.
  type: ApplicationType
}

# An application list order.
input ApplicationListOrder {

  # Field by which to order applications.
  field: ApplicationListOrderField

  # Ordering direction.
  direction: ListOrderDirection
}

# A list of fields by which to order an application list.
enum ApplicationListOrderField {

  # Order applications by type.
  TYPE

  # Order applications by display order.
  DISP_ORDER

  # Order applications by name.
  NAME

  # Order applications by description.
  DESCRIPTION

  # Order applications by active status.
  ACTIVE
}

# An application parameter list order.
input ApplicationParameterListOrder {

  # Field by which to order application parameters.
  field: ApplicationParameterListOrderField

  # Ordering direction.
  direction: ListOrderDirection
}

# A list of fields by which to order an application parameter list.
enum ApplicationParameterListOrderField {

  # Order application parameters by name.
  NAME

  # Order application parameters by description.
  DESCRIPTION

  # Order application parameters by data type.
  DATATYPE

  # Order application parameters by direction.
  DIRECTION
}

# Create application input.
input CreateApplicationInput {

  # Application assembly class name.
  assemblyClassName: String

  # Application assembly name.
  assemblyName: String

  # Client mutation identifier.
  clientMutationId: String

  # Application content type.
  contentType: String

  # Application context format.
  contextFormat: ApplicationContextFormatType

  # Application description.
  description: String!

  # Application active.
  isActive: Boolean!

  # Application default.
  isDefault: Boolean

  # Application schema definition embedded.
  isSchemaDefinitionEmbedded: Boolean

  # Application method.
  method: String

  # Application name.
  name: String!

  # Application password.
  password: String

  # Application token.
  token: String

  # Application type.
  type: ApplicationType!

  # Application URL.
  url: String

  # Application username.
  username: String

  # Application WSDL.
  wsdl: String
}

# Create application result.
type CreateApplicationResult {

  # Client mutation identifier.
  clientMutationId: String

  # Application that has been created.
  application: Application!
}

# Update application input.
input UpdateApplicationInput {

  # Application assembly class name.
  assemblyClassName: String

  # Application assembly name.
  assemblyName: String

  # Client mutation identifier.
  clientMutationId: String

  # Application content type.
  contentType: String

  # Application context format.
  contextFormat: ApplicationContextFormatType

  # Application description.
  description: String

  # Application identifier.
  id: ID!

  # Application active.
  isActive: Boolean

  # Application default.
  isDefault: Boolean

  # Application schema definition.
  isSchemaDefinitionEmbedded: Boolean

  # Application method.
  method: String

  # Application name.
  name: String

  # Application password.
  password: String

  # Application URL.
  url: String

  # Application username.
  username: String

  # Application WSDL.
  wsdl: String
}

# Update application result.
type UpdateApplicationResult {

  # Application that has been updated.
  application: Application!

  # Client mutation identifier.
  clientMutationId: String
}

# Delete application input.
input DeleteApplicationInput {

  # Client mutation identifier.
  clientMutationId: String

  # Application identifier.
  id: ID!
}

# Delete application result.
type DeleteApplicationResult {

  # Client mutation identifier.
  clientMutationId: String
}

# Add an application parameter input.
input AddApplicationParameterInput {

  # Application identifier.
  applicationId: ID!

  # Client mutation identifier.
  clientMutationId: String

  # Application parameter data type.
  dataType: WorkflowDataType!

  # Application parameter description.
  description: String!

  # Application parameter direction.
  direction: ParameterDirection!

  # Application parameter default.
  isDefault: Boolean!

  # Application parameter required.
  isRequired: Boolean!

  # Application parameter name.
  name: String!
}

# Add an application parameter result.
type AddApplicationParameterResult {

  # Client mutation identifier.
  clientMutationId: String

  # Application parameter that has been added.
  parameter: ApplicationParameter!
}

# Remove an application parameter input.
input RemoveApplicationParameterInput {

  # Client mutation identifier.
  clientMutationId: String

  # Application parameter identifier.
  id: ID!
}

# Remove an application parameter result.
type RemoveApplicationParameterResult {

  # Client mutation identifier.
  clientMutationId: String
}

`;
