"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * upload.ts
 * File upload management
 */
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const file_url_1 = __importDefault(require("file-url"));
const file_uri_to_path_1 = __importDefault(require("file-uri-to-path"));
const util_1 = require("util");
const config_1 = __importDefault(require("../config"));
const logger_1 = require("../utils/logger");
const config = config_1.default.getInstance();
const fileUploadPath = config.graphql.fileUploadPath;
function handleUploadParameters(parameters, requestUuid) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!parameters) {
            return;
        }
        let promises = [];
        for (var index = 0; index < parameters.length; index++) {
            if (parameters[index].fileValue && parameters[index].fileValue.upload) {
                const { createReadStream, filename, mimetype } = yield parameters[index].fileValue.upload;
                const stream = createReadStream();
                const fileExtension = path.extname(filename);
                const uploadedFilePath = path.join(fileUploadPath, `${requestUuid}_${index}${fileExtension}`);
                parameters[index].fileValue.url = file_url_1.default(uploadedFilePath);
                if (!parameters[index].fileValue.name) {
                    parameters[index].fileValue.name = filename;
                }
                if (!parameters[index].fileValue.description) {
                    parameters[index].fileValue.description = filename;
                }
                if (!parameters[index].fileValue.contentType) {
                    parameters[index].fileValue.contentType = mimetype;
                }
                promises.push(new Promise((resolve, reject) => stream
                    .on('error', (error) => {
                    if (stream.truncated) {
                        fs.unlinkSync(uploadedFilePath);
                    }
                    reject(error);
                })
                    .pipe(fs.createWriteStream(uploadedFilePath))
                    .on('error', (error) => reject(error))
                    .on('finish', () => resolve())));
            }
        }
        yield Promise.all(promises);
    });
}
function handleUploadFile(file, requestUuid) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!file) {
            return;
        }
        const { createReadStream, filename } = yield file.upload;
        const stream = createReadStream();
        const fileExtension = path.extname(filename);
        const uploadedFilePath = path.join(fileUploadPath, `${requestUuid}${fileExtension}`);
        file.url = file_url_1.default(uploadedFilePath);
        return new Promise((resolve, reject) => stream
            .on('error', (error) => {
            if (stream.truncated) {
                fs.unlinkSync(uploadedFilePath);
            }
            reject(error);
        })
            .pipe(fs.createWriteStream(uploadedFilePath))
            .on('error', reject)
            .on('finish', resolve));
    });
}
function invokeMutation(operation, context, args) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!context.isUpload) {
            return context.workflow.service.invoke(operation, context.workflow.context, args);
        }
        createUploadFolder(fileUploadPath);
        if (args.input.parameters) {
            yield handleUploadParameters(args.input.parameters, context.uuid);
            args.input.parameters
                .filter((fileParam) => fileParam.fileValue)
                .forEach((parameter) => {
                if (parameter.fileValue.upload &&
                    parameter.fileValue.url &&
                    fs.existsSync(file_uri_to_path_1.default(parameter.fileValue.url))) {
                    parameter.fileValue.size = fs.statSync(file_uri_to_path_1.default(parameter.fileValue.url)).size;
                }
            });
        }
        else if (args.input.xpdl) {
            yield handleUploadFile(args.input.xpdl, context.uuid);
        }
        else {
            yield handleUploadFile(args.input.xmlDefinition, context.uuid);
        }
        try {
            return yield context.workflow.service.invoke(operation, context.workflow.context, args);
        }
        finally {
            if (args.input.parameters) {
                deleteTemporaryFiles(args.input.parameters);
            }
            else if (args.input.xpdl) {
                deleteTemporaryFile(args.input.xpdl.url);
            }
            else {
                deleteTemporaryFile(args.input.xmlDefinition.url);
            }
        }
    });
}
exports.invokeMutation = invokeMutation;
function createUploadFolder(fileUploadPath) {
    return __awaiter(this, void 0, void 0, function* () {
        let targetPath;
        // Create the upload folder and its parents if they don't exist
        if (!fs.existsSync(fileUploadPath)) {
            // UNC remote path
            if (fileUploadPath.startsWith('\\\\')) {
                const mkdir = util_1.promisify(fs.mkdir);
                const remotePath = fileUploadPath.replace(/\\\\/g, '');
                targetPath = remotePath.split('\\');
                targetPath[0] = `\\\\${targetPath[0]}`;
                for (var j = 2; j <= targetPath.length; j++) {
                    const folder = targetPath.slice(0, j).join('\\');
                    if (!fs.existsSync(folder)) {
                        yield mkdir(folder);
                    }
                }
            }
            else { // Local Path
                targetPath = fileUploadPath.split('\\');
                for (var i = 1; i <= targetPath.length; i++) {
                    const folder = targetPath.slice(0, i).join('/');
                    if (!fs.existsSync(folder)) {
                        fs.mkdirSync(folder);
                    }
                }
            }
        }
    });
}
function deleteTemporaryFiles(parameters) {
    if (!parameters) {
        return;
    }
    parameters.map((parameter) => {
        if (parameter.fileValue && parameter.fileValue.upload && parameter.fileValue.url) {
            deleteTemporaryFile(parameter.fileValue.url);
        }
    });
}
function deleteTemporaryFile(filePath) {
    const path = file_uri_to_path_1.default(filePath);
    if (!fs.existsSync(path)) {
        return;
    }
    try {
        fs.unlinkSync(path);
    }
    catch (_) {
        logger_1.logger('ERROR', `Unexpected error during file upload management. Temporary file ${path} deletion has failed.`);
    }
}
