"use strict";
/**
 * hooksController.ts
 * Contains the WebHooks API endpoints.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = __importDefault(require("express"));
const v1_1 = __importDefault(require("uuid/v1"));
const express_async_handler_1 = __importDefault(require("express-async-handler"));
const path = __importStar(require("path"));
const url_1 = __importDefault(require("url"));
const workflowservice_1 = __importDefault(require("../services/workflow/workflowservice"));
const logger_1 = require("../utils/logger");
const config_1 = __importDefault(require("../config"));
const upload_1 = require("../upload");
const config = config_1.default.getInstance();
const router = express_1.default.Router();
// Workflow service creation
const workflowService = new workflowservice_1.default(config.workflowService);
// Retrieves the operation body
function getOperationBody(req, res) {
    return __awaiter(this, void 0, void 0, function* () {
        const requestContentType = req.headers['content-type'].toLowerCase();
        if (requestContentType.includes('application/x-www-form-urlencoded')) {
            if (!req.body.payload) {
                logger_1.logger('ERROR', 'Request ' + req.uuid + ' the payload key is not defined');
                throw new Error('The payload key is not defined.');
            }
            try {
                return JSON.parse(req.body.payload);
            }
            catch (e) {
                logger_1.logger('ERROR', 'Request ' + req.uuid + ' invalid JSON in payload');
                throw new Error('Invalid JSON in payload.');
            }
        }
        if (requestContentType.includes('application/json')) {
            return req.body;
        }
        if (requestContentType.includes('multipart/form-data')) {
            try {
                req.isUpload = true;
                return yield upload_1.processUpload(req, res, {
                    maxFileSize: config.workflowService.maxInputFileSize,
                    maxFiles: config.hooks.maxUploadFiles,
                    fileUploadPath: config.hooks.fileUploadPath
                });
            }
            catch (e) {
                logger_1.logger('ERROR', 'Request ' + req.uuid + ' ' + e.message);
                throw e;
            }
        }
        logger_1.logger('ERROR', 'Request ' + req.uuid + ' the content type not supported');
        throw new Error('The content type not supported.');
    });
}
// Process the workflow operation
function processOperation(req, res, body, token) {
    return __awaiter(this, void 0, void 0, function* () {
        logger_1.logger('INFO', 'Request ' + req.uuid + ' token:' + token + ' operation:' + body.operation);
        if (req.headers['x-wfgen-impersonate-username']) {
            if (!config.hooks.allowImpersonateUser) {
                logger_1.logger('ERROR', 'User impersonation not authorized.');
                res.status(401).json({ error: 'User impersonation not authorized.' });
                return;
            }
            if (!config.hooks.apiKeyEnabled) {
                logger_1.logger('ERROR', 'Hooks api key is disabled: User impersonation not authorized.');
                res.status(401).json({ error: 'Hooks api key is disabled: User impersonation not authorized.' });
                return;
            }
            if (!config.hooks.apiKey) {
                logger_1.logger('ERROR', 'Hooks api key is not defined: User impersonation not authorized.');
                res.status(401).json({ error: 'Hooks api key is not defined: User impersonation not authorized.' });
                return;
            }
            if (req.headers['x-wfgen-hooks-api-key'] !== config.hooks.apiKey) {
                logger_1.logger('ERROR', 'x-wfgen-hooks-api-key is not found or invalid: User impersonation not authorized.');
                res.status(401).json({ error: 'x-wfgen-hooks-api-key is not found or invalid: User impersonation not authorized.' });
                return;
            }
        }
        const context = (process.env.NODE_ENV === 'test' && req.body.context)
            ? req.body.context
            : {
                hookToken: token,
                impersonateUserName: req.headers['x-wfgen-impersonate-username'],
                operationCount: 0,
                operationErrorCount: 0
            };
        if (process.env.NODE_ENV !== 'test' && !config.hooks.operations.includes(body.operation)) {
            logger_1.logger('ERROR', 'Request ' + req.uuid + ' operation not found');
            res.status(404).json({ error: 'The requested operation was not found.' });
            return;
        }
        try {
            if (body.operation === 'sendWorkflowDataFile' || body.operation === 'sendProcessDefinition' || body.operation === 'sendGlobalListDefinition') {
                // Send file operation (used by integration test only)
                yield sendFile(req, res, context, body);
            }
            else {
                // Run standard operations
                res.json(yield workflowService.invoke(body.operation, context, body.args));
            }
        }
        catch (err) {
            logger_1.logger('ERROR', 'Request ' + req.uuid + ' ' + err);
            const httpStatusCode = err.statusCode || 500;
            // Workflow service authentication error code 401 is replaced by a 404.
            res.status(httpStatusCode === 401 ? 404 : httpStatusCode).json({ error: err.message });
            return;
        }
        finally {
            if (req.isUpload) {
                if (body.args.input.parameters) {
                    upload_1.deleteTemporaryFiles(body.args.input.parameters);
                }
                else if (body.args.input.xpdl) {
                    upload_1.deleteTemporaryFile(body.args.input.xpdl.url);
                }
                else {
                    upload_1.deleteTemporaryFile(body.args.input.xmlDefinition.url);
                }
            }
        }
    });
}
// Send file operation (used by integration test only)
function sendFile(req, res, context, body) {
    return __awaiter(this, void 0, void 0, function* () {
        const response = {
            set: (data, callback) => {
                res.set(data.name, data.value);
                callback();
            },
            write: (data, callback) => {
                res.write(data);
                callback();
            }
        };
        body.args.response = response;
        var fileSize = yield workflowService.invoke(body.operation, context, body.args);
        logger_1.logger('INFO', `Request ${req.uuid} ${body.operation}: file sent: id: ${body.args.id} size: ${fileSize} bytes`);
    });
}
// Hooks POST endpoint with the token specified in the URL
// The asyncHandler middleware wraps the express route to handle rejected promises.
router.post('/:token', express_async_handler_1.default((req, res) => __awaiter(this, void 0, void 0, function* () {
    res.set('Content-Type', 'application/json');
    req.uuid = v1_1.default();
    logger_1.logger('INFO', 'New request ' + req.uuid);
    try {
        var body = yield getOperationBody(req, res);
    }
    catch (err) {
        res.status(400).json({ error: err.message });
        return;
    }
    if (!body) {
        logger_1.logger('ERROR', 'Request ' + req.uuid + ' JSON body is null');
        res.status(400).json({ error: 'Invalid JSON.' });
        return;
    }
    if (!body.operation) {
        logger_1.logger('ERROR', 'Request ' + req.uuid + ' missing operation');
        res.status(400).json({ error: 'An operation is required.' });
        return;
    }
    if (typeof body.operation !== 'string') {
        logger_1.logger('ERROR', 'Request ' + req.uuid + ' invalid operation value');
        res.status(400).json({ error: 'Invalid operation value.' });
        return;
    }
    yield processOperation(req, res, body, req.params.token);
})));
// Hooks POST endpoint using with the operation name in the URL
// The asyncHandler middleware wraps the express route to handle rejected promises.
router.post('/operations/:operation', express_async_handler_1.default((req, res) => __awaiter(this, void 0, void 0, function* () {
    res.set('Content-Type', 'application/json');
    req.uuid = v1_1.default();
    logger_1.logger('INFO', 'New request ' + req.uuid);
    const operation = req.params.operation;
    if (!operation) {
        logger_1.logger('ERROR', 'Request ' + req.uuid + ' missing operation');
        res.status(400).json({ error: 'An operation is required.' });
        return;
    }
    try {
        var body = yield getOperationBody(req, res);
    }
    catch (err) {
        res.status(400).json({ error: err.message });
        return;
    }
    if (!body) {
        logger_1.logger('ERROR', 'Request ' + req.uuid + ' JSON body is null');
        res.status(400).json({ error: 'Invalid JSON.' });
        return;
    }
    // The requested operation must be the operation defined in the payload body.
    if (!body.operation) {
        body.operation = operation;
    }
    else if (body.operation !== operation) {
        const message = 'The operation value in the payload is not the requested operation.';
        logger_1.logger('ERROR', `Request ${req.uuid} ${message}`);
        res.status(400).json({ error: message });
        return;
    }
    yield processOperation(req, res, body, req.headers['x-wfgen-hooktoken']);
})));
// OpenAPI v2 public endpoint
// The Swagger file is dynamically populated
router.get('/openapi/v2', (_, res) => {
    res.set('Content-Type', 'application/json');
    const swagger = require(path.join(__dirname, '..', '..', 'templates', 'swagger.json'));
    const url = url_1.default.parse(config.applicationUrl, true);
    const npmPackage = require(path.join(__dirname, '..', '..', 'package.json'));
    swagger.info.version = npmPackage.version;
    swagger.host = url.host;
    swagger.basePath = `${url.pathname}/hooks/operations`;
    swagger.schemes = [url.protocol ? url.protocol.slice(0, -1) : undefined];
    res.send(swagger);
});
exports.default = router;
