"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * workflowservice.ts
 * Workflow Service
 */
const util_1 = __importDefault(require("util"));
const edge_js_1 = __importDefault(require("edge-js"));
// Use bluebird Promise to fix:
// https://github.com/tjanczuk/edge/issues/485
// https://github.com/Microsoft/vscode/issues/1746
const bluebird_1 = __importDefault(require("bluebird"));
class WorkflowServiceError extends Error {
    constructor(statusCode, message) {
        super(message);
        this.statusCode = statusCode;
    }
}
exports.WorkflowServiceError = WorkflowServiceError;
// Use as an immutable Map
exports.ExceptionStatusCode = new Map([
    ['System.Exception', 500],
    ['System.Configuration.ConfigurationErrorsException', 500],
    ['System.ArgumentException', 400],
    ['System.InvalidOperationException', 500],
    ['System.IO.FileNotFoundException', 400],
    ['System.TimeoutException', 408],
    ['System.UnauthorizedAccessException', 403],
    ['System.Security.Authentication.AuthenticationException', 401],
    ['System.Data.DuplicateNameException', 409]
]);
/**
 *  Represents a Workflow Service
 */
class WorkflowService {
    /**
     * Creates a new instance of WorkflowService.
     * @param options A service configuration.
     */
    constructor(options) {
        this.options = options;
        this.isStarted = false;
        this.isStarting = false;
        this.isStartError = false;
        this.defaultExceptionStatusCode = 500;
        // Load the workflow service assembly
        this.createWorkflowServiceFacade = edge_js_1.default.func({
            assemblyFile: this.options.serviceAssemblyFilePath,
            typeName: 'Advantys.Workflow.ServiceFacade',
            methodName: 'CreateProxyAsync'
        });
    }
    /**
     * Starts the workflow service.
     * @return A promise.
     */
    start() {
        this.isStarting = true;
        return new bluebird_1.default((resolve, reject) => {
            this.createWorkflowServiceFacade(this.options, (error, result) => {
                if (error) {
                    this.isStarted = false;
                    this.isStarting = false;
                    this.isStartError = true;
                    reject(error);
                    return;
                }
                this.workflowFacade = result;
                this.isStarted = true;
                this.isStarting = false;
                this.isStartError = false;
                resolve();
            });
        });
    }
    /**
     * Invokes a workflow operation asynchronously.
     * @param operation An operation to execute.
     * @param context A workflow context.
     * @param args Arguments.
     * @return A promise.
     */
    invoke(operation, context, args) {
        return this.invokeOperationWithRetry(operation, context, args);
    }
    /**
     * Invokes a workflow operation asynchronously with retries
     * @param operation An operation to execute.
     * @param context A workflow context.
     * @param args Arguments.
     * @param retriedCount Current number of retries.
     * @return A promise.
     */
    invokeOperationWithRetry(operation, context, args, retriedCount = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            // The service is ready to invoke operation
            if (this.isStarted) {
                return this.invokeOperation(operation, context, args);
            }
            // The service is in error
            if (this.isStartError) {
                return bluebird_1.default.reject(new Error('Workflow Service start failed.'));
            }
            // Start the service if needed
            if (!this.isStarting) {
                try {
                    yield this.start();
                    this.options.logHandler('Workflow Service is started');
                }
                catch (err) {
                    this.options.logHandler(`Workflow Service start error: ${err}`);
                }
            }
            // Retry the operation
            retriedCount++;
            if (this.options.debugEnabled) {
                this.options.logHandler(`Workflow Service: invokeWithRetry: retry count: ${retriedCount}`);
            }
            if (retriedCount > this.options.maxOperationRetries) {
                this.options.logHandler('Workflow Service: invokeWithRetry: error: Workflow Service is not started: max retries reached');
                throw new WorkflowServiceError(this.defaultExceptionStatusCode, 'Workflow Service is not started.');
            }
            yield bluebird_1.default.delay(this.options.operationRetryDelay);
            return this.invokeOperationWithRetry(operation, context, args, retriedCount);
        });
    }
    /**
     * Invokes a workflow operation asynchronously.
     * @param operation An operation to execute.
     * @param context A workflow context.
     * @param args Arguments.
     * @return A promise.
     */
    invokeOperation(operation, context, args) {
        const operationContext = {};
        if (this.options.debugEnabled) {
            this.options.logHandler(util_1.default.format('Workflow Service: invokeOperation: %s context: %j args:%j', operation, context, args));
        }
        return new bluebird_1.default((resolve, reject) => {
            if (context.operationCount === this.options.maxOperationCount) {
                reject(new Error('The maximum number of operation per request has been reached.'));
                return;
            }
            context.operationCount++;
            if (context.authUserName) {
                operationContext.authUserName = context.authUserName;
            }
            if (context.impersonateUserName) {
                operationContext.impersonateUserName = context.impersonateUserName;
            }
            if (context.hookToken) {
                operationContext.hookToken = context.hookToken;
            }
            if (context.scimToken) {
                operationContext.scimToken = context.scimToken;
            }
            if (context.appClientId) {
                operationContext.appClientId = context.appClientId;
            }
            this.workflowFacade.invoke({
                operation: operation,
                context: operationContext,
                args: args
            }, (error, result) => {
                if (error) {
                    const errorStatusCode = exports.ExceptionStatusCode.get(error.name);
                    if (this.options.debugEnabled) {
                        this.options.logHandler(`Workflow Service: invokeOperation: error: ${error.message} Service stack: ${error.StackTrace}`);
                    }
                    context.operationErrorCount++;
                    error.statusCode = errorStatusCode || this.defaultExceptionStatusCode;
                    reject(error);
                    return;
                }
                resolve(result);
            });
        });
    }
}
exports.default = WorkflowService;
