"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * directory.ts
 * Contains the logic of a directory service.
 */
const mem_1 = __importDefault(require("mem"));
const user_1 = require("./user");
const group_1 = require("./group");
const patchoperations_1 = require("./patchoperations");
const search_1 = __importDefault(require("../models/search"));
const usertranslator_1 = __importDefault(require("./translator/usertranslator"));
const grouptranslator_1 = __importDefault(require("./translator/grouptranslator"));
const injector_1 = require("../utils/injector");
const array_1 = require("../utils/array");
const config_1 = __importDefault(require("../config"));
const resource_1 = require("./resource");
const config = config_1.default.getInstance();
/**
 * Represents an error thrown by the directory.
 */
class DirectoryError extends Error {
    /**
     * Instanciates a new directory error.
     * @param message A human readable message.
     * @param statusCode The status code of the error.
     */
    constructor(message, statusCode = 400) {
        super(message);
        this.statusCode = statusCode;
    }
}
exports.DirectoryError = DirectoryError;
/**
 * Represents a directory service.
 */
let Directory = class Directory {
    /**
     * Creates an instance of Directory.
     * @param baseUrl The base url treated by the directory service.
     * @param token The token found in the request.
     * @param workflowService The injected workflow service class.
     */
    constructor(baseUrl, token, workflowService) {
        this.baseUrl = baseUrl;
        this.wfgOperationContext = {
            scimToken: token,
            operationCount: 0,
            operationErrorCount: 0
        };
        this.workflowService = workflowService;
        if (config.app.cacheEnabled === true) {
            this.getInfo = mem_1.default(this.getInfo, {
                maxAge: config.app.cacheMaxAge
            });
        }
    }
    /**
     * Creates a new user from predefined attributes.
     * @param userAttributes User attributes for the new user.
     * @return A promise.
     */
    createUser(userAttributes) {
        return __awaiter(this, void 0, void 0, function* () {
            // Exceptionally, we'll accept a value that is not strictly a user here to
            // reuse the translator object instead of translating the attributes one by
            // one.
            const user = new user_1.UserResource(userAttributes);
            const wfgUser = new usertranslator_1.default(user).translate();
            Object.cleanProperties(user);
            Object.cleanProperties(wfgUser);
            const directoryInfo = yield this.getInfo();
            wfgUser.directoryId = directoryInfo.id;
            const result = yield this.workflowService.invoke('createUser', this.wfgOperationContext, {
                input: wfgUser
            });
            user.id = result.userId;
            user.addUrlLocation(this.baseUrl);
            return user;
        });
    }
    /**
     * Creates a groups from its attributes.
     * @param groupAttributes The attributes to create the group from.
     * @return A promise.
     */
    createGroup(groupAttributes) {
        return __awaiter(this, void 0, void 0, function* () {
            // Exceptionally, we'll accept a value that is not strictly a group here to
            // reuse the translator object instead of translating the attributes one by
            // one.
            const group = new group_1.GroupResource(groupAttributes);
            const wfgGroup = new grouptranslator_1.default(group).translate();
            Object.cleanProperties(group);
            Object.cleanProperties(wfgGroup);
            const directoryInfo = yield this.getInfo();
            wfgGroup.directoryId = directoryInfo.id;
            const groupCreationResult = yield this.workflowService.invoke('createGroup', this.wfgOperationContext, {
                input: wfgGroup
            });
            if (group.members) {
                yield this.workflowService.invoke('addUsersToGroup', this.wfgOperationContext, {
                    input: {
                        id: groupCreationResult.groupId,
                        userIds: group.members.map(m => m.value)
                    }
                });
            }
            group.id = groupCreationResult.groupId;
            group.addUrlLocation(this.baseUrl);
            return group;
        });
    }
    /**
     * Retrieve an existing user with the option to specify the attributes
     * needed.
     * @param userId The id of the existing user to retrieve.
     * @param attributes A list of attributes that will be included in the resource.
     * @param excludedAttributes A list of attributes that will be excluded from the resource.
     * @return A promise.
     */
    getUser(userId, attributes, excludedAttributes) {
        return __awaiter(this, void 0, void 0, function* () {
            const [directoryInfo, result] = yield Promise.all([
                this.getInfo(),
                this.workflowService.invoke('getUser', this.wfgOperationContext, { id: userId })
            ]);
            if (directoryInfo.id !== result.directoryId) {
                throw new DirectoryError('The user does not exists in this directory.', 404);
            }
            if (!user_1.WFGUser.validate(result)) {
                throw new Error('Invalid object returned from the facade.');
            }
            const translator = new usertranslator_1.default(result);
            const user = translator.translate();
            if (attributes) {
                user.includesOnly(...attributes);
            }
            else if (excludedAttributes) {
                user.excludes(...excludedAttributes);
            }
            user.checkAddBusinessExtension();
            Object.cleanProperties(user);
            user.addUrlLocation(this.baseUrl);
            return user;
        });
    }
    /**
     * Retrieve an existing group with the option to specify the attributes.
     * needed.
     * @param groupId The id of the group to retrieve.
     * @param attributes A list of attributes to be included in the resource.
     * @param excludedAttributes A list of attributes to be excluded from the resource.
     * @return A promise that resolves with the group.
     */
    getGroup(groupId, attributes, excludedAttributes) {
        return __awaiter(this, void 0, void 0, function* () {
            const [directoryInfo, wfgGroup] = yield Promise.all([
                this.getInfo(),
                this.workflowService.invoke('getGroup', this.wfgOperationContext, {
                    id: groupId
                })
            ]);
            if (directoryInfo.id !== wfgGroup.directoryId) {
                throw new DirectoryError('The requested group does not exists in this directory.', 404);
            }
            if (!group_1.WFGGroup.validate(wfgGroup)) {
                throw new Error('Invalid object returned from the facade.');
            }
            const wfgUsers = yield this.workflowService.invoke('getUsers', this.wfgOperationContext, {
                groupId: wfgGroup.id
            });
            const translator = new grouptranslator_1.default(wfgGroup);
            const group = translator.translate();
            group.members = wfgUsers.items
                .filter(user => user_1.WFGUser.validate(user))
                .map((user) => new usertranslator_1.default(user).translate())
                .map(user => {
                user.addUrlLocation(this.baseUrl);
                return {
                    value: user.id,
                    display: user.displayName,
                    '$ref': user.meta.location
                };
            });
            if (attributes) {
                group.includesOnly(...attributes);
            }
            else if (excludedAttributes) {
                group.excludes(...excludedAttributes);
            }
            Object.cleanProperties(group);
            group.addUrlLocation(this.baseUrl);
            return group;
        });
    }
    /**
     * Modifies a user accordingly to specified operations.
     * @param userId The id of an existing user.
     * @param operationsToExecute An array of operations to execute on the user.
     * @return A promise. UserResource if resolved.
     */
    partialUpdateUser(userId, operationsToExecute) {
        return __awaiter(this, void 0, void 0, function* () {
            const [directoryInfo, wfgUser] = yield Promise.all([
                this.getInfo(),
                this.workflowService.invoke('getUser', this.wfgOperationContext, {
                    id: userId
                })
            ]);
            if (directoryInfo.id !== wfgUser.directoryId) {
                throw new DirectoryError('The user cannot be found in this directory.', 404);
            }
            if (!user_1.WFGUser.validate(wfgUser)) {
                throw new Error('Invalid object returned from the facade.');
            }
            let translator = new usertranslator_1.default(wfgUser);
            let scimUser = translator.translate();
            for (const op of operationsToExecute) {
                if (!op.op || !patchoperations_1.Validators.isScimOperation(op)) {
                    throw new patchoperations_1.PatchOpMissingError();
                }
                const operation = patchoperations_1.PatchOperationFactory.fromOperation(op);
                scimUser = operation.execute(scimUser);
            }
            scimUser.checkAddBusinessExtension();
            translator = new usertranslator_1.default(scimUser);
            yield this.workflowService.invoke('updateUser', this.wfgOperationContext, {
                input: translator.translate()
            });
            Object.cleanProperties(scimUser);
            scimUser.addUrlLocation(this.baseUrl);
            return scimUser;
        });
    }
    /**
     * Modifies a group according to specified operations.
     * @param {String} groupId The id of an existing group.
     * @param {Array<Object>} operationsToExecute An array of operations to execute on the group.
     * @return {Promise<GroupResource>} A promise. GroupResource if resolved.
     * @memberOf Directory
     */
    partialUpdateGroup(groupId, operationsToExecute) {
        return __awaiter(this, void 0, void 0, function* () {
            let group = yield this.getGroup(groupId);
            const memberIds = group.members ? group.members.map(m => m.value) : [];
            for (const op of operationsToExecute) {
                if (!op.op || !patchoperations_1.Validators.isScimOperation(op)) {
                    throw new patchoperations_1.PatchOpMissingError();
                }
                const operation = patchoperations_1.PatchOperationFactory.fromOperation(op);
                group = operation.execute(group);
                group.members = [].concat.apply([], group.members);
                const keys = {};
                group.members = group.members.filter(m => {
                    const seen = keys[m.value];
                    keys[m.value] = true;
                    return typeof seen === 'undefined';
                });
            }
            const newMembersIds = group.members.map(m => m.value);
            const commonIds = array_1.intersect(memberIds, newMembersIds);
            const toDeleteIds = memberIds.filter(id => !commonIds.includes(id));
            const toAddIds = newMembersIds.filter(id => !commonIds.includes(id));
            const newGroup = new grouptranslator_1.default(group).translate();
            delete newGroup.users;
            const [updateRes] = yield Promise.all([
                this.workflowService.invoke('updateGroup', this.wfgOperationContext, {
                    input: newGroup
                }),
                toAddIds.length > 0
                    ? this.workflowService.invoke('addUsersToGroup', this.wfgOperationContext, {
                        input: {
                            id: group.id,
                            userIds: toAddIds
                        }
                    })
                    : null,
                toDeleteIds.length > 0
                    ? this.workflowService.invoke('removeUsersFromGroup', this.wfgOperationContext, {
                        input: {
                            id: group.id,
                            userIds: toDeleteIds
                        }
                    })
                    : null
            ]);
            const usersRes = yield this.workflowService.invoke('getUsers', this.wfgOperationContext, {
                groupId: updateRes.groupId
            });
            group.members = usersRes.items
                .filter(u => user_1.WFGUser.validate(u))
                .map((u) => new usertranslator_1.default(u).translate())
                .map(u => {
                u.addUrlLocation(this.baseUrl);
                return {
                    value: u.id,
                    display: u.displayName,
                    '$ref': u.meta.location
                };
            });
            Object.cleanProperties(group);
            group.addUrlLocation(this.baseUrl);
            return group;
        });
    }
    /**
     * Deletes a user from the directory.
     * @param userId The id of an existing user.
     * @return A promise.
     */
    deleteUser(userId) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.workflowService.invoke('deleteUser', this.wfgOperationContext, {
                input: {
                    id: userId
                }
            });
        });
    }
    /**
     * Deletes a group from the directory.
     * @param groupId The id of the group.
     * @return A promise.
     */
    deleteGroup(groupId) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.workflowService.invoke('deleteGroup', this.wfgOperationContext, {
                input: {
                    id: groupId
                }
            });
        });
    }
    /**
     * Search in the directory for resource according to parameters.
     * @param schema The schema to restrict directory search.
     * @param filter The search filter string not parsed.
     * @param sortBy The attribute name from which to sort by.
     * @param sortOrder The order of the sort.
     * @param startIndex The start index of the pagination.
     * @param count The maximum number of results to return.
     * @param attributes The attributes to include.
     * @param excludedAttributes The attributes to exclude.
     * @return A promise with the results.
     */
    search(schema, filter, sortBy, sortOrder, startIndex = 0, count = 0, attributes, excludedAttributes) {
        return __awaiter(this, void 0, void 0, function* () {
            const search = search_1.default.fromSchema(schema);
            search.context = this.wfgOperationContext;
            search.directoryId = (yield this.getInfo()).id;
            if (filter) {
                search.filter = filter;
            }
            if (sortBy) {
                search.sortBy = sortBy;
            }
            if (sortOrder) {
                search.sortOrder = sortOrder;
            }
            if (startIndex) {
                search.startIndex = startIndex;
            }
            if (count) {
                search.count = count;
            }
            if (attributes) {
                search.attributes = attributes;
            }
            if (excludedAttributes) {
                search.excludedAttributes = excludedAttributes;
            }
            const results = yield search.execute();
            results.forEach(e => {
                if (!resource_1.Resource.validate(e)) {
                    throw new Error('Invalid object returned from search operation.');
                }
            });
            return results;
        });
    }
    getInfo() {
        return __awaiter(this, void 0, void 0, function* () {
            const results = yield this.workflowService.invoke('getDirectories', this.wfgOperationContext, {
                filter: {
                    synchronizationConnector: 'AZURE_SCIM_V2'
                }
            });
            if (results.totalCount <= 0) {
                throw new Error('No SCIM directory found on WorkflowGen server.');
            }
            return results.items[0];
        });
    }
};
Directory = __decorate([
    injector_1.Injector.withServices
], Directory);
exports.default = Directory;
