"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = __importDefault(require("../constants"));
const logger_1 = require("./logger");
/**
 * Contains methods to add details to HTTP errors
 */
class ErrorHelper {
    /**
     * Creates an instance of ErrorHelper.
     * @param status The error http status code.
     * @param detail The detail message of the error.
     * @param scimType The scim type of the error.
     */
    constructor(status, detail, scimType) {
        this.status = status;
        this.detail = detail;
        this.scimType = scimType;
        this.schemas = [constants_1.default.schemas.error];
    }
    /**
     * Adds an error to the HTTP response according to the SCIM spec.
     * @param res The HTTP response to add the error to.
     * @param status The HTTP status code to use.
     * @param detail Human readable text explaining the error.
     * @param scimType The scimType of the HTTP error. Only used for 400 errors.
     */
    static addErrorResponse(res, status, detail, scimType) {
        res.status(status).json(new ErrorHelper(status, detail, scimType));
    }
    /**
     * Adds the 404 error code to the HTTP response.
     * @param res The HTTP response to add the error to.
     */
    static sendErrorNotFound(res) {
        this.addErrorResponse(res, 404, 'Requested resource was not found');
    }
    /**
     * Adds an ID missing error to the response. Uses HTTP error code 400.
     * @param res The HTTP response to add the error to.
     */
    static sendIdMissing(res) {
        this.addErrorResponse(res, 400, 'No resource ID was provided', 'invalidValue');
    }
    /**
     * Adds an incorrect schema error to the response.
     * @param res The HTTP response to add the error to.
     */
    static sendIncorrectSchema(res) {
        this.addErrorResponse(res, 400, 'Incorrect SCIM schema in the request body', 'invalidValue');
    }
    /**
     * Adds an invalid path error for PATCH remove of a complex attribute.
     * @param res The HTTP response to add the error to.
     */
    static sendPatchRemoveComplexAttributeError(res) {
        this.addErrorResponse(res, 400, 'Tried to remove a complex attribute.', 'invalidPath');
    }
    /**
     * Adds an operation array required error for the PATCH operation
     * @param res The HTTP response to add the error to.
     */
    static sendPatchOperationsMissing(res) {
        this.addErrorResponse(res, 400, 'Missing the required PATCH Operations array.', 'invalidValue');
    }
    /**
     * Adds an op member required error for the PATCH operation
     * @param res The HTTP response to add the error to.
     */
    static sendPatchOpMissing(res) {
        this.addErrorResponse(res, 400, 'Missing the required PATCH op member.', 'invalidValue');
    }
    /**
     * Adds a generic invalid value error.
     * @param res The HTTP response to add the error to.
     * @param message The error's message.
     */
    static sendInvalidValue(res, message) {
        this.addErrorResponse(res, 400, message, 'invalidValue');
    }
    /**
     * Adds a invalid path error for a PATCH request
     * @param res The HTTP response to add the error to.
     */
    static sendNoPath(res) {
        this.addErrorResponse(res, 400, 'Remove operation has unspecified path', 'noTarget');
    }
    /**
     * Adds a noTarget error for a PATCH replace request with filter
     * @param res The HTTP response to add the error to.
     */
    static sendNoPathReplace(res) {
        this.addErrorResponse(res, 400, 'Replace operation filter found no element', 'noTarget');
    }
    /**
     * Adds a missing value error for a PATCH request
     * @param res The HTTP response to add the error to.
     */
    static sendNoValue(res) {
        this.addErrorResponse(res, 400, 'No value was provided for an operation', 'invalidValue');
    }
    /**
     * Adds a mutability error for a PATCH operation
     * @param res The HTTP response to add the error to.
     */
    static sendImmutablePatchError(res) {
        this.addErrorResponse(res, 400, 'Patch operation tried to modify an immutable attribute.', 'mutability');
    }
    /**
     * Adds a unauthorized error for a request.
     * @param res The HTTP response to add the error to.
     */
    static sendUnauthorizedError(res) {
        this.addErrorResponse(res, 401, 'This operation is not authorized on this resource.');
    }
    /**
     * Sends the SCIM error relating to uniqueness of a resource.
     * @param res the HTTP response to add the error to.
     */
    static sendUniquenessError(res) {
        this.addErrorResponse(res, 409, 'Element exists in the database', 'uniqueness');
    }
    /**
     * Adds an internal server error to the response (HTTP 500).
     * @param res The HTTP response to add the error to.
     * @param detail The detail of the error.
     */
    static sendInternalServerError(res, detail) {
        this.addErrorResponse(res, 500, detail || 'Internal server error');
    }
    /**
     * Adds an invalid syntax error (one or more attributes missing)
     * @param res The HTTP response to add the error to.
     */
    static sendInvalidSyntax(res) {
        this.addErrorResponse(res, 400, 'One or more required attributes were not provided.', 'invalidSyntax');
    }
    /**
     * Adds an invalid filter error.
     * @param res The HTTP response to add the error to.
     */
    static sendInvalidFilterError(res) {
        this.addErrorResponse(res, 400, 'The filter provided is invalid.', 'invalidFilter');
    }
    /**
     * Adds a too many results error.
     * @param res The HTTP response to add the error to.
     */
    static sendTooManyQueryResults(res) {
        this.addErrorResponse(res, 400, 'The search request returns too many results.', 'tooMany');
    }
    /**
     * Handles errors returned or thrown by the Workflow Service.
     * @param err The error from the workflow service.
     * @param res The express response object.
     * @param logAction The log action to be passed to the logger.
     * @param directory The directory object that performs the action.
     */
    static handleWorkflowServiceErrors(err, res, logAction, directory) {
        switch (err.statusCode) {
            case 400:
                if (err.message.indexOf('ID') !== -1 &&
                    err.message.indexOf('invalid') !== -1) {
                    logger_1.logToFile(logAction, directory, 'Not found');
                    this.sendErrorNotFound(res);
                    break;
                }
                logger_1.logToFile(logAction, directory, 'Invalid value');
                this.sendInvalidValue(res, err.message);
                break;
            case 404:
                logger_1.logToFile(logAction, directory, 'Resource not found in the directory.');
                this.sendErrorNotFound(res);
                break;
            case 409:
                logger_1.logToFile(logAction, directory, 'Already exists');
                this.sendUniquenessError(res);
                break;
            case 401:
            case 403:
                logger_1.logToFile(logAction, directory, 'Unauthorized access attempt');
                this.sendUnauthorizedError(res);
                break;
            case 500:
                if (err.message.indexOf('does not exist') !== -1) {
                    logger_1.logToFile(logAction, directory, 'Not found');
                    this.sendErrorNotFound(res);
                    break;
                }
                if (err.message.indexOf('already exists') !== -1) {
                    logger_1.logToFile(logAction, directory, 'Already exists');
                    ErrorHelper.sendUniquenessError(res);
                    break;
                }
                logger_1.logToFile(logAction, directory, 'Unexpected error: "%s"', err.message);
                this.sendInternalServerError(res, err.message);
                break;
            default:
                break;
        }
    }
}
exports.default = ErrorHelper;
