"use strict";
/**
 * resource.ts
 * Shared model for SCIM resources.
 */
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Abstract. Represents a SCIM resource.
 */
class Resource {
    /**
     * Creates an instance of Resource.
     */
    constructor(resourceType, obj) {
        this.requiredAttributes = [
            'id',
            'schemas',
            'meta',
            'requiredAttributes'
        ];
        this.meta = {
            resourceType
        };
        if (obj) {
            this.id = obj.id;
            this.schemas = obj.schemas;
            this.externalId = obj.externalId;
        }
    }
    /**
     * Deletes properties that are not needed.
     * @param attributes Attributes that will only appear in the representation.
     * @return Return this instance.
     */
    includesOnly(...attributes) {
        if (attributes.length > 0) {
            Object.keys(this).forEach(prop => {
                if (!attributes.includes(prop) &&
                    this.requiredAttributes.every(element => element !== prop)) {
                    delete this[prop];
                }
            });
        }
        return this;
    }
    /**
     * Deletes the excluded properties from this object.
     * @param attributes Attributes to be excluded.
     * @return Returns this instance.
     */
    excludes(...attributes) {
        attributes.forEach(attr => {
            if (this.hasOwnProperty(attr) &&
                this.requiredAttributes.every(element => element !== attr)) {
                delete this[attr];
            }
        });
        return this;
    }
    /**
     * Overrides json serialization process.
     * @return This object but clean.
     */
    toJSON() {
        delete this.requiredAttributes;
        Object.cleanProperties(this);
        return this;
    }
    /**
     * Validates if the type of an object is a resource.
     * @param obj The object to validate.
     */
    static validate(obj) {
        return 'id' in obj &&
            'schemas' in obj;
    }
}
exports.Resource = Resource;
class WFGResource {
    constructor(id) {
        this.id = id;
    }
    /**
     * Validates if the type of an object is a WFGResource.
     * @param obj The object to validate.
     */
    static validate(obj) {
        return 'id' in obj;
    }
}
exports.WFGResource = WFGResource;
