"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var Search_1;
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * search.ts
 * Model for the SCIM search options.
 */
const filter_1 = __importDefault(require("./filter"));
const constants_1 = __importDefault(require("../models/constants"));
const usertranslator_1 = __importDefault(require("./translator/usertranslator"));
const grouptranslator_1 = __importDefault(require("./translator/grouptranslator"));
const injector_1 = require("../utils/injector");
class TooManyResultsError extends Error {
}
exports.TooManyResultsError = TooManyResultsError;
class SearchError extends Error {
}
exports.SearchError = SearchError;
class InvalidSortOrderError extends SearchError {
    constructor() {
        super('The sort order is invalid. It should be ASCENDING or DESCENDING');
    }
}
exports.InvalidSortOrderError = InvalidSortOrderError;
class InvalidFilterError extends SearchError {
}
exports.InvalidFilterError = InvalidFilterError;
class InvalidStartIndexError extends SearchError {
    constructor() {
        super('The start index is not a number.');
    }
}
exports.InvalidStartIndexError = InvalidStartIndexError;
class InvalidCountError extends SearchError {
    constructor() {
        super('The count is not a number.');
    }
}
exports.InvalidCountError = InvalidCountError;
class UnauthorizedSearchError extends SearchError {
    constructor() {
        super('Unauthorized search on this resource.');
    }
}
exports.UnauthorizedSearchError = UnauthorizedSearchError;
/**
 * SCIM Search model.
 */
let Search = Search_1 = class Search {
    /**
     * Creates an instance of Search.
     */
    constructor(workflowService) {
        this.workflowService = workflowService;
        this.directoryId = '';
        this._sortOrder = 'ascending';
        this._startIndex = 0;
        this._count = 0;
        this._wfgOperationContext = null;
        this.searchUsers = false;
        this.searchGroups = false;
    }
    set sortOrder(value) {
        if (value !== 'ascending' && value !== 'descending') {
            throw new InvalidSortOrderError();
        }
        this._sortOrder = value;
    }
    set context(context) {
        this._wfgOperationContext = context;
    }
    set filter(value) {
        try {
            this._filter = new filter_1.default(value);
        }
        catch (err) {
            throw new InvalidFilterError();
        }
    }
    set startIndex(value) {
        let max = Math.max(value, 1);
        if (isNaN(max)) {
            throw new InvalidStartIndexError();
        }
        this._startIndex = max;
    }
    get startIndex() {
        return this._startIndex;
    }
    get count() {
        return this._count;
    }
    set count(value) {
        let max = Math.max(value, 0);
        if (isNaN(max)) {
            throw new InvalidCountError();
        }
        this._count = max;
    }
    /**
     * Creates an instance of Search with a schema.
     * @param resourceSchema The SCIM schema to search on.
     * @return The search model.
     */
    static fromSchema(resourceSchema, /* WorkflowService */ workflowService) {
        const search = new Search_1(workflowService);
        search.resourceSchema = resourceSchema;
        if (resourceSchema === constants_1.default.schemas.user) {
            search.searchUsers = true;
        }
        if (resourceSchema === constants_1.default.schemas.group) {
            search.searchGroups = true;
        }
        return search;
    }
    /**
     * Searches user resources.
     * @param callback The function to call back with the results.
     */
    search(callback) {
        let additionalProcessing = (results) => results;
        let filter;
        let translator;
        let methodName;
        let TranslatorClass; // typeof Translator
        if (this.searchUsers && !this.searchGroups) {
            translator = new usertranslator_1.default();
            methodName = 'getUsers';
            TranslatorClass = usertranslator_1.default;
            if (this._filter) {
                try {
                    filter = this._filter.getJavaScriptObject();
                }
                catch (e) {
                    throw new InvalidFilterError();
                }
            }
        }
        else if (this.searchGroups && !this.searchUsers) {
            translator = new grouptranslator_1.default();
            methodName = 'getGroups';
            TranslatorClass = grouptranslator_1.default;
            if (this._filter) {
                try {
                    filter = this._filter.getJavaScriptObject();
                }
                catch (e) {
                    throw new InvalidFilterError();
                }
            }
        }
        if (filter && translator) {
            // Transform the SCIM attribute into a WFG attribute
            for (const prop in filter) {
                const [scimProp, wfgProp] = translator.translateAttribute(prop);
                if (wfgProp && wfgProp !== prop) {
                    filter[wfgProp] = filter[prop];
                    delete filter[prop];
                }
            }
            if (filter.hasOwnProperty('userName')) {
                filter.userNames = [filter.userName];
                delete filter.userName;
            }
            if (filter.hasOwnProperty('name')) {
                additionalProcessing = function (results) {
                    return __awaiter(this, void 0, void 0, function* () {
                        if (results.items.length > 0) {
                            const groupUsers = yield this.workflowService.invoke('getUsers', this._wfgOperationContext, {
                                groupId: results.items[0].id
                            });
                            results.items[0].users = groupUsers.items;
                        }
                        return results;
                    });
                }.bind(this);
            }
            if (filter.hasOwnProperty('id') && filter.hasOwnProperty('managerId')) {
                filter.ids = [filter.id];
                additionalProcessing = function (results) {
                    results.items = results.items.filter((item) => item.managerId === this.managerId);
                    return results;
                }.bind({ managerId: filter.managerId });
                delete filter.id;
                delete filter.managerId;
            }
            if (filter.hasOwnProperty('id') && filter.hasOwnProperty('users')) {
                filter.ids = [filter.id];
                additionalProcessing = (results) => __awaiter(this, void 0, void 0, function* () {
                    results.items = results.items.filter((g) => g.id === filter.id);
                    if (results.items.length > 0) {
                        const users = yield this.workflowService.invoke('getUsers', this._wfgOperationContext, {
                            groupId: results.items[0].id
                        });
                        results.items = results.items.filter((g) => !!users.items.find((u) => u.id === filter.users));
                        results.items[0].users = users.items;
                    }
                    return results;
                });
                delete filter.id;
                delete filter.users;
            }
            if (filter.hasOwnProperty('systemIdentifier')) {
                filter.systemIdentifiers = [filter.systemIdentifier];
                additionalProcessing = (results) => __awaiter(this, void 0, void 0, function* () {
                    if (this.searchGroups) {
                        for (let i = 0; i < results.items.length; i++) {
                            const users = yield this.workflowService.invoke('getUsers', this._wfgOperationContext, {
                                groupId: results.items[i].id
                            });
                            results.items[i].users = users.items;
                        }
                    }
                    return results;
                });
                delete filter.systemIdentifier;
            }
        }
        this.workflowService.invoke(methodName, this._wfgOperationContext, filter)
            .then(additionalProcessing)
            .then(results => {
            const elements = results.items
                .filter((item) => item.directoryId === this.directoryId)
                .map((item) => {
                const element = new TranslatorClass(item).translate();
                if (this.searchGroups && element.members) {
                    element.members = element.members.map((m) => ({
                        value: m.id,
                        display: m.displayName
                    }));
                }
                if (this.attributes) {
                    element.includesOnly(...this.attributes);
                }
                else if (this.excludedAttributes) {
                    element.excludes(...this.excludedAttributes);
                }
                return element;
            });
            callback(elements);
        })
            .catch(err => callback(undefined, err));
    }
    /**
     * Executes the search operation.
     * @return A promise indicating success or failure.
     */
    execute() {
        return new Promise((resolve, reject) => {
            if (this.resourceSchema === constants_1.default.schemas.user &&
                !this.searchUsers) {
                throw new UnauthorizedSearchError();
            }
            if (this.resourceSchema === constants_1.default.schemas.group &&
                !this.searchGroups) {
                throw new UnauthorizedSearchError();
            }
            if (!this.resourceSchema &&
                !(this.searchUsers && this.searchGroups)) {
                throw new UnauthorizedSearchError();
            }
            this.search((results, err) => {
                if (err) {
                    reject(err);
                    return;
                }
                resolve(results);
            });
        });
    }
};
Search = Search_1 = __decorate([
    injector_1.Injector.withServices
], Search);
exports.default = Search;
