"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * usertranslator.ts
 * Contains the definition of the UserTranslator class.
 */
const user_1 = require("../user");
const constants_1 = __importDefault(require("../constants"));
const translator_1 = __importDefault(require("./translator"));
const ACCEPTED_PHONE_NUMBER_TYPES = new Set([
    'phone', 'mobile', 'fax', 'pager'
]);
/**
 * Concrete User translator
 */
class UserTranslator extends translator_1.default {
    getSCIMAttributesMap() {
        return new Map([
            ['id', 'id'],
            ['userName', 'userName'],
            ['name', 'lastName'],
            ['name.familyName', 'lastName'],
            ['name.givenName', 'firstName'],
            ['displayName', 'commonName'],
            ['title', 'jobTitle'],
            ['locale', 'defaultLanguage'],
            ['timezone', 'defaultTimeZoneId'],
            ['externalId', 'systemIdentifier'],
            ['active', 'isActive'],
            ['emails.value', 'email'],
            ['phoneNumbers.value', ''],
            ['addresses.streetAddress', 'postalAddress'],
            ['addresses.locality', 'city'],
            ['addresses.region', 'state'],
            ['addresses.postalCode', 'postalCode'],
            ['addresses.country', 'country'],
            ['employeeNumber', 'employeeNumber'],
            ['organization', 'company'],
            ['department', 'department'],
            ['manager', 'managerId']
        ]);
    }
    translateToWFG(object) {
        let user = new user_1.WFGUser(object.id, typeof object.name === 'object'
            ? object.name.familyName
            : object.name);
        user.id = object.id;
        user.userName = object.userName;
        user.systemIdentifier = object.externalId;
        if (typeof object.name === 'object') {
            user.firstName = object.name.givenName;
        }
        user.commonName = object.displayName;
        user.jobTitle = object.title;
        if (object.locale) {
            user.defaultLanguage = object.locale;
        }
        user.isActive = object.active;
        if (object.emails && object.emails.length > 0) {
            user.email = typeof object.emails[0] === 'string'
                ? object.emails[0]
                : object.emails[0].value;
        }
        object.phoneNumbers && object.phoneNumbers.forEach((num) => {
            if (typeof num === 'string' && ACCEPTED_PHONE_NUMBER_TYPES.has(num)) {
                user.phone = num;
            }
            else if (ACCEPTED_PHONE_NUMBER_TYPES.has(num.type || '')) {
                const numValued = num;
                if (numValued.type) {
                    user[numValued.type] = numValued.value;
                }
            }
        });
        if (object.addresses && object.addresses.length > 0) {
            const workAddress = object.addresses.find(addr => addr.type === 'work');
            const otherAddress = object.addresses.find(addr => addr.type === 'other');
            if (workAddress) {
                user.postalAddress = workAddress.streetAddress;
                user.city = workAddress.locality;
                user.state = workAddress.region;
                user.postalCode = workAddress.postalCode;
                user.country = workAddress.country;
            }
            if (otherAddress) {
                user.office = otherAddress.formatted;
            }
        }
        if (object.hasOwnProperty(constants_1.default.schemas.extensions.user.business)) {
            const businessProperties = object[constants_1.default.schemas.extensions.user.business];
            user.employeeNumber = businessProperties.employeeNumber;
            user.company = businessProperties.organization;
            user.department = businessProperties.department;
            if (businessProperties.manager) {
                user.managerId = businessProperties.manager.id;
            }
        }
        Object.cleanProperties(user, false);
        return user;
    }
    translateToSCIM(object) {
        let user = new user_1.UserResource();
        user.id = object.id;
        user.externalId = object.systemIdentifier;
        user.userName = object.userName;
        if (object.lastName && object.firstName) {
            user.name = {
                givenName: object.firstName,
                familyName: object.lastName
            };
        }
        else if (object.lastName) {
            user.name = object.lastName;
        }
        user.displayName = object.commonName;
        user.title = object.jobTitle;
        user.locale = object.defaultLanguage;
        if (typeof object.isActive !== 'undefined') {
            user.active = object.isActive;
        }
        user.emails = [];
        user.phoneNumbers = [];
        user.addresses = [];
        if (object.email) {
            // string & ValuedAttribute<string> type is impossible to represent.
            // Therefore, we choose one and tel the compiler that we know what we
            // are doing.
            user.emails.push({
                value: object.email,
                type: 'work',
                primary: true
            });
        }
        ACCEPTED_PHONE_NUMBER_TYPES.forEach(type => {
            if (object.hasOwnProperty(type) && object[type]) {
                user.phoneNumbers.push({
                    value: object[type],
                    type
                });
            }
        });
        if (object.postalAddress ||
            object.city ||
            object.state ||
            object.postalCode ||
            object.country) {
            user.addresses.push({
                type: 'work',
                streetAddress: object.postalAddress,
                locality: object.city,
                region: object.state,
                postalCode: object.postalCode,
                country: object.country
            });
        }
        if (object.office) {
            user.addresses.push({
                type: 'other',
                formatted: object.office
            });
        }
        if (object.employeeNumber ||
            object.company ||
            object.department ||
            object.managerId) {
            user[constants_1.default.schemas.extensions.user.business] = {
                employeeNumber: object.employeeNumber,
                organization: object.company,
                department: object.department,
                manager: {
                    id: object.managerId
                }
            };
        }
        Object.cleanProperties(user, false);
        return user;
    }
    isSCIM(obj) {
        return user_1.UserResource.validate(obj);
    }
    isWFG(obj) {
        return user_1.WFGUser.validate(obj);
    }
}
exports.default = UserTranslator;
