"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * user.ts
 * Constains the definition of a user of the business domain.
 */
const constants_1 = __importDefault(require("../models/constants"));
const resource_1 = require("./resource");
/**
 * Represents a response to send to the user.
 */
class UserResource extends resource_1.Resource {
    constructor(obj = undefined) {
        super('User', obj);
        this.schemas = [constants_1.default.schemas.user];
        if (obj) {
            for (let prop of Object.getOwnPropertyNames(obj)) {
                this[prop] = obj[prop];
            }
        }
    }
    /**
     * Adds the business schema extension if necessary
     */
    checkAddBusinessExtension() {
        if (this.employeeNumber ||
            this.costCenter ||
            this.organization ||
            this.division ||
            this.department ||
            this.manager) {
            this.schemas.push(constants_1.default.schemas.extensions.user.business);
        }
    }
    /**
     * Adds the display name of the manager to the representation.
     * @param displayName The display name of the manager.
     */
    addManagerInfo(displayName) {
        if (Object.isNullOrUndefined(this.manager)) {
            this.manager = { displayName: '' };
        }
        this.manager.displayName = displayName;
    }
    /**
     * Adds the full url location to the representation.
     * @param baseUrl The base url of the resource.
     */
    addUrlLocation(baseUrl) {
        if (this.manager && !Object.isNullOrUndefined(this.manager.id)) {
            this.manager.$ref = `${baseUrl}/${this.manager.id}`;
        }
        this.meta.location = `${baseUrl}/${this.id}`;
    }
    /**
     * Format multiple names and titles into a single one.
     * @param honorificPrefix The honorific prefix of the user.
     * @param givenName The given name of the user.
     * @param middleName The middle name of the user.
     * @param lastName The last name of the user.
     * @param honorificSuffix The honorific suffix of the user.
     * @return The formatted name altogether.
     */
    formattedName(honorificPrefix = undefined, givenName = undefined, middleName = undefined, lastName = undefined, honorificSuffix = undefined) {
        let formatted = '';
        let space = ' ';
        if (honorificPrefix) {
            formatted += honorificPrefix + space;
        }
        if (givenName) {
            formatted += givenName + space;
        }
        if (middleName) {
            formatted += middleName + space;
        }
        if (lastName) {
            formatted += lastName + space;
        }
        if (honorificSuffix) {
            formatted += honorificSuffix + space;
        }
        return formatted.substr(0, formatted.length - 1);
    }
    /**
     * Format the address from various values.
     * @param streetAddress The street address.
     * @param locality The locality.
     * @param region The region.
     * @param postalCode The postal code.
     * @param country The country.
     * @return The formatted address.
     */
    formattedAddress(streetAddress = undefined, locality = undefined, region = undefined, postalCode = undefined, country = undefined) {
        let formatted = '';
        let space = ' ';
        let thirdPartPresent = (region || postalCode || country);
        if (locality || thirdPartPresent) {
            formatted += streetAddress || '\n';
        }
        else {
            formatted += streetAddress || '';
        }
        if (locality && thirdPartPresent) {
            formatted += '\n' + locality + ', ';
        }
        else if (locality) {
            formatted += '\n' + locality + space;
        }
        if (region) {
            formatted += region + space;
        }
        if (postalCode) {
            formatted += postalCode + space;
        }
        if (country) {
            formatted += country + space;
        }
        return formatted.substr(0, formatted.length - 1);
    }
}
exports.UserResource = UserResource;
/**
 * Represents a WorkflowGen user.
 */
class WFGUser extends resource_1.WFGResource {
    constructor(id, lastName) {
        super(id);
        this.lastName = lastName;
    }
    static validate(obj) {
        const superValidated = super.validate(obj);
        if (!superValidated) {
            return false;
        }
        return 'lastName' in obj;
    }
}
exports.WFGUser = WFGUser;
