"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * server.ts
 * Server initialization
 */
const express_1 = __importDefault(require("express"));
const bodyParser = __importStar(require("body-parser"));
const cors_1 = __importDefault(require("cors"));
const v1_1 = __importDefault(require("uuid/v1"));
const altlogger_1 = require("./utils/altlogger");
const config_1 = __importDefault(require("./config"));
const error_1 = __importDefault(require("./models/helpers/error"));
const injector_1 = require("./utils/injector");
const workflowservice_1 = __importDefault(require("./services/workflow/workflowservice"));
const path = __importStar(require("path"));
require("./utils/extensions"); // Load extensions
const config = config_1.default.getInstance();
const ROOT_URL = '(\\/\\w+)?(\\/scim)?\\/scim';
const USERS_URL_REGEX = new RegExp(`${ROOT_URL}/Users`);
const GROUPS_URL_REGEX = new RegExp(`${ROOT_URL}/Groups`);
const app = express_1.default();
if (config.test && config.test.runMode !== 'app') {
    require('../test/stub').stubWorkflowService();
}
else {
    injector_1.Injector.configure({
        classes: [workflowservice_1.default],
        args: [
            [config.workflowService]
        ]
    });
}
altlogger_1.FileLogger.configure({
    formatter: options => {
        const action = (options.meta && options.meta.action) || "GENERAL" /* General */;
        const directory = (options.meta && options.meta.directory) || 'DIRECTORY';
        return `${options.timestamp.toJSON()};${directory};${action};${options.message}`;
    },
    getLogFilePath: meta => {
        const today = new Date();
        const year = today.getFullYear();
        const month = today.getMonth() + 1;
        const day = today.getDate();
        const formattedDate = `${year}${month.toString().length === 1 ? '0' + month : month}${day.toString().length === 1 ? '0' + day : day}`;
        return path.join(config.app.logsPath, `${formattedDate}_000000_${meta.directory}.log`);
    }
});
altlogger_1.logger.console.info('Application NODE_ENV: %s', process.env.NODE_ENV);
altlogger_1.logger.console.info(`Workflow service debug enabled: ${config.workflowService.debugEnabled}`);
// Enable Cross Origin Resource Sharing
app.use('*', cors_1.default());
// Parse application/json
app.use(bodyParser.json({
    type: 'application/scim+json'
}));
// Catch json syntax error
app.use(function (error, req, res, next) {
    if (error instanceof SyntaxError) {
        altlogger_1.logger.console.error('Invalid JSON');
        error_1.default.sendInvalidSyntax(res);
        return;
    }
    next();
});
// Authorization middleware
app.use(function (req, res, next) {
    res.set('Content-Type', 'application/scim+json');
    req.uuid = v1_1.default();
    altlogger_1.logger.console.info(`New request ${req.uuid}`);
    const authorizationHeader = req.header('authorization'); // e.g. Authorization: Bearer <token>
    let token;
    if (authorizationHeader) {
        const authComponents = authorizationHeader.split(' ');
        if (authComponents.length !== 2) {
            altlogger_1.logger.console.error(`Request ${req.uuid} invalid bearer token`);
            error_1.default.sendUnauthorizedError(res);
            return;
        }
        if (authComponents[0] !== 'Bearer') {
            altlogger_1.logger.console.error(`Request ${req.uuid} invalid bearer token`);
            error_1.default.sendUnauthorizedError(res);
            return;
        }
        token = authComponents[1];
        if (token === '') {
            altlogger_1.logger.console.error(`Request ${req.uuid} missing bearer token`);
            error_1.default.sendUnauthorizedError(res);
            return;
        }
    }
    else {
        altlogger_1.logger.console.error(`Request ${req.uuid} missing authorization header`);
        error_1.default.sendUnauthorizedError(res);
        return;
    }
    req.authToken = token;
    next();
});
// Router configuration
const users_1 = __importDefault(require("./controllers/users"));
const groups_1 = __importDefault(require("./controllers/groups"));
app.use(USERS_URL_REGEX, users_1.default);
app.use(GROUPS_URL_REGEX, groups_1.default);
// Start Express Server
if (process.env.NODE_ENV !== 'test' || (config.test && config.test.runMode !== 'app')) {
    app.listen(config.app.port, function () {
        altlogger_1.logger.console.info(`SCIM server running on port ${config.app.port}`);
    });
}
else {
    module.exports = app;
}
