"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * altlogger.ts
 * Contains the implementation of the Logger class.
 */
const path = __importStar(require("path"));
const format_1 = __importDefault(require("format"));
const lockfile = __importStar(require("proper-lockfile"));
const fs = __importStar(require("fs"));
const child_process_1 = require("child_process");
var FormatOptions;
(function (FormatOptions) {
    class Builder {
        setLevel(value) {
            this.level = value;
            return this;
        }
        setMessage(value) {
            this.message = value;
            return this;
        }
        setTimestamp(value) {
            this.timestamp = value;
            return this;
        }
        setMeta(value) {
            this.meta = value;
            return this;
        }
        /**
         * Builds the options object.
         * @throws {Error} Thrown when level, message or timestamp are undefined.
         */
        build() {
            if (!this.level || typeof this.message === 'undefined' || !this.timestamp) {
                throw new Error('You must define at least the level, message and timestamp.');
            }
            return {
                level: this.level,
                message: this.message,
                timestamp: this.timestamp,
                meta: this.meta
            };
        }
    }
    FormatOptions.Builder = Builder;
})(FormatOptions || (FormatOptions = {}));
/**
 * Represents an abstract logger.
 */
class Logger {
    /**
     * Parses the args and builds an object to passe it to the handlers.
     *
     * The first argument can be either a Date representing the timetamp or a
     * String representing the level. If the first argument is a Date, the second
     * must be a String representing the level.
     *
     * Le last argument can be an Object and is optional. This object is whatever
     * you want to pass to the handlers to format or output.
     *
     * The argument after the level is the log message or the format of the log
     * message as a String. The other arguments after that are the corresponding
     * variables that you want to format in the format string.
     *
     * This function uses the unix format string in the likes of printf and sprintf.
     */
    log(...args) {
        let builder = new FormatOptions.Builder();
        if (args.length > 0) {
            if (args[0] instanceof Date) {
                builder
                    .setTimestamp(args[0])
                    .setLevel(args[1]);
                args.splice(0, 2);
            }
            else {
                builder
                    .setLevel(args[0])
                    .setTimestamp(new Date());
                args.splice(0, 1);
            }
            if (typeof args[args.length - 1] === 'object') {
                builder.setMeta(args[args.length - 1]);
                args.pop();
            }
        }
        if (args.length > 1) {
            const format = args[0];
            args.splice(0, 1);
            builder.setMessage(format_1.default(format, ...args));
        }
        else if (args.length > 0) {
            builder.setMessage(args[0]);
        }
        else {
            builder.setMessage('');
        }
        const options = builder.build();
        return this.handleOutput(this.formatter(options), options);
    }
    /**
     * Logs an error message.
     * @see {@link Logger#_log} for precision on args and return value.
     */
    error(...args) {
        return this.log("error" /* error */, ...args);
    }
    /**
     * Logs a warning message.
     * @see {@link Logger#_log} for precision on args and return value.
     */
    warn(...args) {
        return this.log("warn" /* warning */, ...args);
    }
    /**
     * Logs an info message.
     * @see {@link Logger#_log} for precision on args and return value.
     */
    info(...args) {
        return this.log("info" /* info */, ...args);
    }
    /**
     * Logs a debug message.
     * @see {@link Logger#_log} for precision on args and return value.
     */
    debug(...args) {
        return this.log("debug" /* debug */, ...args);
    }
    /**
     * Configures the logger according to the app' specific needs.
     * @param {object} options The configurable options for the logger.
     * @param {function} [options.formatter] A function that formats the message to log.
     */
    static configure(options) {
        if (options.formatter) {
            this.prototype.formatter = options.formatter;
        }
    }
}
/**
 * Concrete logger that outputs to the console.
 */
class ConsoleLogger extends Logger {
    handleOutput(formattedText, options) {
        global.console.log(formattedText);
        return Promise.resolve();
    }
    formatter(options) {
        return `${options.timestamp.toJSON()} ${(options.level && options.level.toUpperCase()) || 'INFO'} ${options.message}`;
    }
}
exports.ConsoleLogger = ConsoleLogger;
class FileLogger extends Logger {
    log(...args) {
        const _super = Object.create(null, {
            log: { get: () => super.log }
        });
        return __awaiter(this, void 0, void 0, function* () {
            const timestamp = new Date();
            let logFilePath;
            if (args.length > 0 && typeof args[args.length - 1] === 'object') {
                const meta = args[args.length - 1];
                logFilePath = this.getLogFilePath(meta);
            }
            else {
                logFilePath = this.getLogFilePath({});
            }
            const logsPath = path.dirname(logFilePath);
            if (!fs.existsSync(logsPath)) {
                mkdir(logsPath);
            }
            if (!fs.existsSync(logFilePath)) {
                fs.writeFileSync(logFilePath, '');
            }
            yield lockfile.lock(logFilePath, {
                retries: {
                    retries: 10,
                    minTimeout: 500
                }
            });
            yield _super.log.call(this, ...[timestamp].concat(args));
            lockfile.unlockSync(logFilePath);
        });
    }
    handleOutput(formattedText, options) {
        return new Promise((resolve, reject) => {
            fs.appendFile(this.getLogFilePath(options.meta), '\n' + formattedText, (err) => {
                if (err) {
                    reject(err);
                    return;
                }
                resolve();
            });
        });
    }
    formatter(options) {
        return `${options.timestamp.toJSON()} ${(options.level && options.level.toUpperCase()) || 'INFO'} ${options.message}`;
    }
    /**
     * Get the default log file path. Configurable.
     * @see {@link FileLogger#configure} for how to configure.
     */
    getLogFilePath(meta) {
        const today = new Date();
        const year = today.getFullYear();
        const month = today.getMonth() + 1;
        const day = today.getDate();
        const formattedDate = `${year}${month.toString().length === 1 ? '0' + month : month}${day.toString().length === 1 ? '0' + day : day}`;
        return path.join(path.normalize('./logs'), `${formattedDate}.log`);
    }
    /**
     * @see {@link Logger#configure} for more options.
     */
    static configure(options) {
        super.configure(options);
        if (options.getLogFilePath) {
            this.prototype.getLogFilePath = options.getLogFilePath;
        }
    }
}
exports.FileLogger = FileLogger;
/**
 * Create a new directory in the file system.
 */
function mkdir(path) {
    const nodeVersionMajor = process.versions.node
        .split('.')
        .slice(0, 1)
        .map(versionNumber => parseInt(versionNumber, undefined))
        .pop();
    if (nodeVersionMajor && nodeVersionMajor >= 10) {
        fs.mkdirSync(path, { recursive: true });
        return;
    }
    child_process_1.execSync(`${process.platform === 'win32' ? 'MD' : 'mkdir -p'} "${path}"`);
}
var logger;
(function (logger) {
    logger.console = new ConsoleLogger();
    logger.file = new FileLogger();
})(logger = exports.logger || (exports.logger = {}));
