"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * expressions.ts
 * Typed expressions for filtering.
 */
const operators_1 = require("./operators");
/**
 * General expression class.
 */
class Expression {
    /**
     * Creates an instance of Expression.
     * @param left The left expression.
     * @param right The right expression.
     * @param operator The operand to apply between the expressions.
     */
    constructor(left = undefined, right = undefined, operator) {
        this.left = left;
        this.right = right;
        this.operator = operator;
    }
    /**
     * Applies the expression to an object.
     * @param obj The object to apply the expression to.
     * @return The filtered object.
     */
    apply(obj) {
        return this.test(obj);
    }
}
exports.Expression = Expression;
/**
 * A compound expression. Left and right operand can be other expressions.
 */
class CompoundExpression extends Expression {
    constructor(left = undefined, right, operator) {
        super(left, right, operator);
    }
    /**
     * Tests a compound expression tree.
     * @param elem The element to apply boolean operators on.
     * @return The boolean result.
     */
    test(elem) {
        const left = this.left;
        const right = this.right;
        if (operators_1.GroupingOperator.validate('token', this.operator)) {
            const leftResult = left.test(elem);
            return operators_1.GroupingOperator.getBooleanFunction(leftResult, right.test(elem), this.operator);
        }
        else if (operators_1.LogicalOperator.validate(this.operator)) {
            return operators_1.LogicalOperator.getBooleanFunction(left.test(elem), right.test(elem), this.operator);
        }
        return false;
    }
}
exports.CompoundExpression = CompoundExpression;
/**
 * A simple expression. Left and right operand are attributes.
 */
class SimpleExpression extends Expression {
    constructor(left, right = undefined, operator) {
        super(left, right, operator);
    }
    /**
     * Tests a simple expression.
     * @param elem The element to apply tests on.
     * @return The boolean result.
     */
    test(elem) {
        return operators_1.AttributeOperator.getBooleanFunction(this.left, this.operator, this.right, elem);
    }
}
exports.SimpleExpression = SimpleExpression;
