"use strict";
/**
 * operators.ts
 * Typed operators for filtering.
 */
Object.defineProperty(exports, "__esModule", { value: true });
var GroupingOperator;
(function (GroupingOperator) {
    GroupingOperator["OP"] = "(";
    GroupingOperator["CP"] = ")";
    GroupingOperator["NP"] = "NOT(";
})(GroupingOperator = exports.GroupingOperator || (exports.GroupingOperator = {}));
var AttributeOperator;
(function (AttributeOperator) {
    /**
     * Checks if the token is an attribute operator.
     * @param token The token to check.
     * @return If the token is an attribute operator.
     */
    function validate(token) {
        const upperToken = token.toUpperCase();
        return upperToken === 'EQ' ||
            upperToken === 'CO' ||
            upperToken === 'SW' ||
            upperToken === 'EW' ||
            upperToken === 'PR' ||
            upperToken === 'GT' ||
            upperToken === 'GE' ||
            upperToken === 'LT' ||
            upperToken === 'LE' ||
            upperToken === 'NE';
    }
    AttributeOperator.validate = validate;
    function getBooleanFunction(attribute, operator, valueToTest = undefined, object) {
        const value = object[attribute];
        switch (operator) {
            case 'EQ':
                return String(value) === valueToTest;
            case 'NE':
                return String(value) !== valueToTest;
            case 'CO':
                if (!(typeof value === 'string')) {
                    throw new UnsupportedOperationError();
                }
                return value.includes(valueToTest);
            case 'SW':
                if (!(typeof value === 'string')) {
                    throw new UnsupportedOperationError();
                }
                return value.startsWith(valueToTest);
            case 'EW':
                if (!(typeof value === 'string')) {
                    throw new UnsupportedOperationError();
                }
                return value.endsWith(valueToTest);
            case 'PR':
                return typeof value !== 'undefined';
            case 'GT':
                if (typeof value === 'boolean') {
                    throw new UnsupportedOperationError();
                }
                return value > valueToTest;
            case 'GE':
                if (typeof value === 'boolean') {
                    throw new UnsupportedOperationError();
                }
                return value >= valueToTest;
            case 'LT':
                if (typeof value === 'boolean') {
                    throw new UnsupportedOperationError();
                }
                return value < valueToTest;
            case 'LE':
                if (typeof value === 'boolean') {
                    throw new UnsupportedOperationError();
                }
                return value <= valueToTest;
        }
    }
    AttributeOperator.getBooleanFunction = getBooleanFunction;
})(AttributeOperator = exports.AttributeOperator || (exports.AttributeOperator = {}));
var LogicalOperator;
(function (LogicalOperator) {
    function validate(token) {
        return token === 'AND' ||
            token === 'OR';
    }
    LogicalOperator.validate = validate;
    /**
     * Logical test between two booleans.
     * @param left The left boolean.
     * @param right The right boolean.
     * @param operator The operator to use.
     * @return Test result between left and right.
     */
    function getBooleanFunction(left, right, operator) {
        switch (operator) {
            case 'AND':
                return left && right;
            case 'OR':
                return left || right;
        }
    }
    LogicalOperator.getBooleanFunction = getBooleanFunction;
})(LogicalOperator = exports.LogicalOperator || (exports.LogicalOperator = {}));
(function (GroupingOperator) {
    const GroupingOperatorMap = new Map([
        ['OP', '('],
        ['CP', ')'],
        ['NP', 'NOT(']
    ]);
    /**
     * Checks if the symbol is a grouping operator.
     * @param symbolToTest The symbol to check.
     * @return If the symbol is an attribute operator.
     */
    function validate(type, token) {
        switch (type) {
            case 'symbol':
                return !!GroupingOperatorMap.get(token);
            case 'token':
                return GroupingOperatorMap.has(token);
        }
    }
    GroupingOperator.validate = validate;
    /**
     * Returns the grouping operator from a symbol.
     * @param symbolToFind The symbol to find the operator from.
     * @return The operator.
     */
    function fromSymbol(symbol) {
        // tslint:disable: no-unnecessary-qualifier
        switch (symbol) {
            case GroupingOperator.CP:
                return GroupingOperator.CP;
            case GroupingOperator.NP:
                return GroupingOperator.OP;
            case GroupingOperator.OP:
                return GroupingOperator.OP;
            default:
                return undefined;
        }
        // tslint:enable: no-unnecessary-qualifier
    }
    GroupingOperator.fromSymbol = fromSymbol;
    /**
     * Tests a grouping operator (i.e. not ()).
     * @param left The left boolean.
     * @param right The right boolean.
     * @param operator The operator to use.
     * @return The grouping result.
     */
    function getBooleanFunction(left, right, operator) {
        switch (operator) {
            // tslint:disable-next-line: no-unnecessary-qualifier
            case GroupingOperator.NP:
                return !right;
            default:
                throw new Error('Unsupported logical operator');
        }
    }
    GroupingOperator.getBooleanFunction = getBooleanFunction;
})(GroupingOperator = exports.GroupingOperator || (exports.GroupingOperator = {}));
class UnsupportedOperationError extends Error {
    constructor() {
        super('This filter operation is not supported');
    }
}
exports.UnsupportedOperationError = UnsupportedOperationError;
