"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * parser.ts
 * Parses tokenized arrays.
 */
const operators_1 = require("./operators");
const expressions_1 = require("./expressions");
const lexer_1 = __importDefault(require("./lexer"));
class InvalidOperatorError extends Error {
}
class InvalidAttributeOperatorError extends InvalidOperatorError {
}
/**
 * Parses a list of tokens.
 */
class FilterParser {
    constructor(lexer) {
        this.lexer = lexer;
    }
    static get Errors() {
        return {
            InvalidOperatorError,
            InvalidAttributeOperatorError
        };
    }
    /**
     * Parse the input tokens.
     * @return The parsed expression.
     */
    parse() {
        return this.leftToRightParse();
    }
    /**
     * Parses the tokens from left to right.
     * @param expressionIn The current expression.
     * @return The next expression.
     */
    leftToRightParse(expressionIn = undefined) {
        let expressionOut = expressionIn;
        while (this.lexer.hasNext()) {
            expressionOut = this.recursiveParse(expressionOut);
        }
        return expressionOut;
    }
    /**
     * Recursively parses an expression.
     * @param expressionIn The expression to parse.
     * @return The next expression.
     */
    recursiveParse(expressionIn = undefined) {
        let expressionOut = expressionIn;
        if (this.lexer.hasNext()) {
            const token = this.lexer.next();
            if (this.isAttributeName(token)) {
                const attributeExpression = this.parseAttributeExpression(token);
                expressionOut = this.recursiveParse(attributeExpression);
            }
            else if (operators_1.LogicalOperator.validate(token)) {
                if (!expressionIn) {
                    throw new Error('No expression was given but a logicial operator was found.');
                }
                const logicalExpression = this.parseCompoundExpression(expressionIn, token);
                expressionOut = this.recursiveParse(logicalExpression);
            }
            else if (operators_1.GroupingOperator.validate('symbol', token)) {
                switch (operators_1.GroupingOperator.fromSymbol(token)) {
                    case operators_1.GroupingOperator.OP:
                        expressionOut = this.parseGroupedExpression(false);
                        break;
                    case operators_1.GroupingOperator.NP:
                        expressionOut = this.parseGroupedExpression(true);
                        break;
                    case operators_1.GroupingOperator.CP:
                        break;
                    default:
                        return expressionIn;
                }
            }
            return expressionOut;
        }
        return expressionIn;
    }
    /**
     * Parses a simple expression between two operands.
     * @param attributeName The attribute to operate on.
     * @return A simple expression between two operands.
     */
    parseAttributeExpression(attributeName) {
        const left = attributeName;
        const operator = this.lexer.next();
        let right;
        if (!operator || !operators_1.AttributeOperator.validate(operator)) {
            throw new InvalidAttributeOperatorError(`Operator "${operator}" is not valid.`);
        }
        if (operator !== 'PR') {
            right = this.lexer.next();
            if (right) {
                right = right.substring(1, right.length - 1);
            }
        }
        return new expressions_1.SimpleExpression(left, right, operator);
    }
    /**
     * Parses a compound expression between two expressions.
     * @param left The left expression.
     * @param operator The operator.
     * @return A compound expression between two expressions.
     */
    parseCompoundExpression(left, operator) {
        const right = this.leftToRightParse();
        if (!right) {
            throw new Error('Failed to parse compound expression.');
        }
        return new expressions_1.CompoundExpression(left, right, operator);
    }
    /**
     * Parses a group of expressions.
     * @param negativeGroup If the group is a not () operator.
     * @return A grouped expression.
     */
    parseGroupedExpression(negativeGroup) {
        let expressionOut;
        let tokens = [];
        let token;
        do {
            token = this.lexer.next();
            if (!token) {
                throw new Error('An undefined token has been found in the middle of the lexer.');
            }
            tokens.push(token);
        } while (token !== ')');
        const previousLexer = this.lexer;
        this.lexer = new lexer_1.default(tokens);
        const groupedExpression = this.leftToRightParse();
        this.lexer = previousLexer;
        if (negativeGroup) {
            const operator = 'NP';
            const right = groupedExpression;
            if (!right) {
                throw new Error('An expression must be given with an NP operator.');
            }
            expressionOut = new expressions_1.CompoundExpression(undefined, right, operator);
        }
        else {
            expressionOut = groupedExpression;
        }
        if (!expressionOut) {
            throw new Error('Failed to parse grouped expression.');
        }
        return expressionOut;
    }
    /**
     * Determines if a token is an attribute.
     * @param token The token to check.
     * @return If the token is an attribute.
     */
    isAttributeName(token) {
        return (!token.startsWith('\'') ||
            !token.endsWith('\'')) &&
            !this.isOperator(token);
    }
    /**
     * Determines if a token is an operator.
     * @param token The token to check.
     * @return If the token is an operator.
     */
    isOperator(token) {
        return operators_1.AttributeOperator.validate(token) ||
            operators_1.GroupingOperator.validate('symbol', token) ||
            operators_1.LogicalOperator.validate(token);
    }
}
exports.default = FilterParser;
