"use strict";
/**
 * injector.ts
 * Contains the definition of the dependency injector.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const getClassInjectRegex = (className) => new RegExp(`\\/\\*\\s?${className}\\s?\\*\\/`);
var Injector;
(function (Injector) {
    let classes = [];
    let args = [];
    function configure(options) {
        if (options.classes) {
            classes = options.classes;
        }
        if (options.args) {
            args = options.args;
        }
    }
    Injector.configure = configure;
    function getArgumentsToPass(passedArguments) {
        const injected = [];
        for (let i = 0; i < classes.length; i++) {
            let injectedArguments = [];
            if (i < args.length) {
                injectedArguments = args[i];
            }
            injected.push(new classes[i](...injectedArguments));
        }
        return passedArguments.concat(injected);
    }
    function instanciateClassToInject(className) {
        const classIndex = classes
            .map(Class => Class.name)
            .indexOf(className);
        const Class = classes[classIndex];
        const classArguments = args[classIndex];
        if (classArguments) {
            return new Class(...classArguments);
        }
        return new Class();
    }
    function withServices(constructor) {
        class WrappedClass extends constructor {
            constructor(...args) {
                const passedArguments = Array.prototype.slice.call(args);
                const argumentsToPass = getArgumentsToPass(passedArguments);
                super(...argumentsToPass);
            }
        }
        const classNameRegexes = classes
            .map(Class => Class.name)
            .map(getClassInjectRegex);
        Object.getOwnPropertyNames(constructor)
            .filter(name => name !== 'constructor' &&
            name !== 'name' &&
            name !== 'length' &&
            name !== 'prototype')
            .forEach(name => {
            // For static functions, we need to check if it needs to have an object
            // injected in its arguments.
            if (typeof constructor[name] === 'function') {
                const signature = constructor[name].toString().split('\\n')[0];
                const wrappedStaticFunc = function () {
                    const passedArguments = Array.prototype.slice.call(arguments);
                    const classesToInject = [];
                    // The function indicates in the signature a comment that contains the exact
                    // name of the class it wants injected.
                    classNameRegexes.forEach(regex => {
                        const matchedclasses = regex.exec(signature);
                        if (matchedclasses && matchedclasses.length > 0) {
                            const matchedClass = matchedclasses[0];
                            const cleanMatchedClass = matchedClass.replace(/[/*\s?]|[*/\s?]/g, '');
                            classesToInject.push(instanciateClassToInject(cleanMatchedClass));
                        }
                    });
                    return constructor[name](...passedArguments.concat(classesToInject));
                };
                // For this kind of dynamic programming, there is no way to define the
                // interface of a static indexer like this line does.
                // @ts-ignore
                WrappedClass[name] = wrappedStaticFunc;
                return;
            }
            // @ts-ignore
            WrappedClass[name] = constructor[name];
        });
        return WrappedClass;
    }
    Injector.withServices = withServices;
})(Injector = exports.Injector || (exports.Injector = {}));
